#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : videos
# @Time         : 2025/6/11 15:13
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import time

from meutils.pipe import *
from meutils.io.files_utils import to_url, to_base64
from meutils.decorators.retry import retrying

from meutils.llm.clients import AsyncClient
from meutils.schemas.openai_types import CompletionRequest
from meutils.schemas.video_types import VideoRequest, SoraVideoRequest

from meutils.db.redis_db import redis_aclient
# from meutils.llm.check_utils import check_token_for_volc
from meutils.llm.check_utils import check_token_for_volc_with_cache as check_token_for_volc

from meutils.config_utils.lark_utils import get_next_token_for_polling, get_series

from fastapi import APIRouter, File, UploadFile, Query, Form, Depends, Request, HTTPException, status, BackgroundTasks

# Please activate
FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Z59Js10DbhT8wdt72LachSDlnlf?sheet=rcoDg7"


# todo 剔除每天超过300w tokens的 确保稳定
async def get_valid_token(tokens: Optional[list] = None, batch_size: Optional[int] = None, seed: int = 0,
                          recheck: bool = False):
    """返回 tokens
    api_key = np.random.choice(api_key.split())

    """
    api_key = tokens or await get_next_token_for_polling(
        feishu_url=FEISHU_URL,
        from_redis=True,
        ttl=24 * 3600,
        check_token=check_token_for_volc
    )
    if isinstance(api_key, list):  # 向下兼容
        api_key = '\n'.join(api_key)

    return api_key

    # tokens = tokens or await get_series(FEISHU_URL, duplicated=True)
    # batch_size = batch_size or 1
    #
    #
    # if seed == 0 and (volc_tokens := await redis_aclient.get(f"volc_tokens")):  # 刷新
    #     volc_tokens = volc_tokens.decode()
    #     if recheck:
    #         valid_tokens = await check_token_for_volc(volc_tokens.split('\n'))
    #
    #         volc_tokens = '\n'.join(valid_tokens)
    #
    #     return volc_tokens
    #
    # valid_tokens = []
    # for token in tokens:
    #     if token := await check_token_for_volc([token]):
    #         valid_tokens += token
    #
    #         logger.debug(valid_tokens)
    #         if len(valid_tokens) == batch_size:
    #             _ = '\n'.join(valid_tokens)
    #             await redis_aclient.set(f"volc_tokens", _, ex=2 * 3600)
    #             # ttl =await redis_aclient.ttl(f"volc_tokens")
    #
    #             return _


# check_image_and_video = partial(check, purpose='video and image')


@retrying()
async def create_task(request: Union[CompletionRequest, VideoRequest, SoraVideoRequest], api_key: Optional[str] = None):
    # api_key = api_key or await get_next_token_for_polling(feishu_url=FEISHU_URL, check_token=check)
    # api_key = api_key or await get_valid_token()
    #
    # api_key = np.random.choice(api_key.split('\n'))  # todo 轮询

    api_key = api_key or await get_next_token_for_polling(
        feishu_url=FEISHU_URL,
        from_redis=True,
        ttl=24 * 3600,
        check_token=check_token_for_volc
    )

    logger.debug(f"api_key: {api_key}")
    if isinstance(request, SoraVideoRequest):  # todo 首尾帧 参考图
        if request.model.startswith("doubao-seedance-1-0-pro"):  # "doubao-seedance-1-0-pro-fast-251015_1080p"
            request.prompt = request.prompt.split('--', maxsplit=1)[0]  # 取消命令行参数
            request.prompt = f"{request.prompt} --rt {request.ratio} --dur {request.seconds}"
            if request.resolution:
                request.prompt += f" --rs {request.resolution}"

            payload = {
                "model": request.model,

                "content": [
                    {
                        "type": "text",
                        "text": request.prompt
                    }
                ]
            }
            if request.input_reference:
                images = await to_base64(request.input_reference)
                if len(images) > 2:
                    images = images[:2]

                # 首尾帧
                for role, image in zip(["first_frame", "last_frame"], images):
                    _ = {
                        "role": role,  # first_frame last_frame reference_image
                        "type": "image_url",
                        "image_url": {
                            "url": image
                        },
                    }
                    payload["content"] += [_]


    elif isinstance(request, VideoRequest):  # 兼容jimeng
        request.prompt = f"{request.prompt.replace('--', '')} --rt {request.aspect_ratio} --dur {request.duration} --rs 1080p"

        payload = {
            "model": "doubao-seedance-1-0-lite-t2v-250428",

            "content": [
                {
                    "type": "text",
                    "text": request.prompt
                }
            ]
        }
        if request.image_url:
            payload = {
                "model": "doubao-seedance-1-0-lite-i2v-250428",

                "content": [
                    {
                        "type": "text",
                        "text": request.prompt
                    },
                    {
                        "type": "image_url",
                        "image_url": {
                            "url": request.image_url
                        }
                    }
                ]
            }
        if request.image_url and request.tail_image_url:
            payload = {
                "model": "doubao-seedance-1-0-lite-i2v-250428",

                "content": [
                    {
                        "type": "text",
                        "text": request.prompt
                    },
                    {
                        "type": "image_url",
                        "image_url": {
                            "url": request.image_url
                        },
                        "role": "first_frame"
                    },
                    {
                        "type": "image_url",
                        "image_url": {
                            "url": request.tail_image_url
                        },
                        "role": "last_frame"
                    }
                ]
            }

        if "-pro" in request.model:
            payload['model'] = "doubao-seedance-1-0-pro-250528"  # 未来注销
    else:

        payload = {
            "model": request.model,
        }

        if hasattr(request, 'content'):
            payload["content"] = request.content

        elif image_urls := request.last_urls.get("image_url"):
            if payload["model"] not in {"doubao-seedance-1-0-lite-i2v-250428", "doubao-seedance-1-0-pro-250528"}:
                payload["model"] = "doubao-seedance-1-0-lite-i2v-250428"

            payload["content"] = [
                {
                    "type": "text",
                    "text": request.last_user_content
                },
                {
                    "type": "image_url",
                    "image_url": {
                        "url": image_urls[-1]
                        # "url": await to_url(image_urls[-1], filename=".png")

                    }
                }]
        else:
            payload["content"] = [
                {
                    "type": "text",
                    "text": request.last_user_content
                }]

    logger.debug(payload)

    client = AsyncClient(base_url="https://ark.cn-beijing.volces.com/api/v3", api_key=api_key)

    response = await client.post(
        path="/contents/generations/tasks",
        cast_to=object,
        body=payload
    )

    if task_id := response.get('id'):
        await redis_aclient.set(task_id, api_key, ex=7 * 24 * 3600)

    return response  # {'id': 'cgt-20250611152553-r46ql'}


async def get_task(task_id: str, api_key: Optional[str] = None):
    if api_key is None:
        token = api_key or await redis_aclient.get(task_id)  # 绑定对应的 token
        api_key = token and token.decode()
        if not token:
            raise HTTPException(status_code=404, detail="TaskID not found")

    client = AsyncClient(base_url="https://ark.cn-beijing.volces.com/api/v3", api_key=api_key)

    response = await client.get(
        path=f"/contents/generations/tasks/{task_id}",
        cast_to=object,
    )

    return response


async def get_task_from_feishu(task_id: Union[str, list], tokens: Optional[list] = None):  # todo: 定时校验
    feishu_url = "https://xchatllm.feishu.cn/sheets/Z59Js10DbhT8wdt72LachSDlnlf?sheet=rcoDg7"
    tokens = tokens or await get_series(feishu_url)

    if isinstance(task_id, str):
        task_ids = [task_id]
    else:
        task_ids = task_id

    for task_id in tqdm(set(task_ids)):
        if not await redis_aclient.get(task_id):
            for api_key in tqdm(tokens):
                client = AsyncClient(base_url="https://ark.cn-beijing.volces.com/api/v3", api_key=api_key)
                try:
                    response = await client.get(
                        path=f"/contents/generations/tasks/{task_id}",
                        cast_to=object,
                    )

                    await redis_aclient.set(task_id, api_key, ex=7 * 24 * 3600)
                    logger.debug(f"{task_id} => {api_key}")
                    break

                except Exception as e:
                    # logger.error(e)
                    continue


# 执行异步函数
if __name__ == "__main__":
    # api_key = "07139a08-e360-44e2-ba31-07f379bf99ed"  # {'id': 'cgt-20250611164343-w2bzq'} todo 过期调用get

    api_key = "c2449725-f758-42af-8f2c-e05b68dd06ad"  # 欠费

    api_key = None

    request = CompletionRequest(
        model="doubao-seedance-1-0-pro-250528",
        # model="doubao-seaweed-241128",
        messages=[
            {"role": "user",
             "content": "无人机以极快速度穿越复杂障碍或自然奇观，带来沉浸式飞行体验  --resolution 1080p  --duration 5 --camerafixed false"}
        ],
    )
    request = VideoRequest(
        model="doubao-seedance-1-0-pro-250528",
        prompt="无人机以极快速度穿越复杂障碍或自然奇观，带来沉浸式飞行体验",
        duration=10
    )
    # r = arun(create_task(request))
    # r = {'id': 'cgt-20250612172542-6nbt2'}

    # arun(get_task(r.get('id')))

    # arun(get_task("cgt-20250707162431-smhwc"))

    # arun(get_task("cgt-20250707160713-j8kll"))

    tokens = """
    
7499ffeb-dbb1-4016-b178-98407c701f09
c4356b58-4aa3-4a52-b907-b40c4dd2e502

    """.split()

    ids = [
        "cgt-20250826185409-d6vb2", "cgt-20250826185409-h8dsf"
    ]

    # arun(get_task_from_feishu(ids, tokens))
    # arun(get_task_from_feishu(ids, ))

    arun(get_valid_token())

"""
{'id': 'cgt-20250613160030-2dvd7',
 'model': 'doubao-seedance-1-0-pro-250528',
 'status': 'running',
 'created_at': 1749801631,
 'updated_at': 1749801631}
 
openai.NotFoundError: Error code: 404 - {'error': {'code': 'ModelNotOpen', 'message': 'Your account 2107813928 has not activated the model doubao-seedance-1-0-pro. Please activate the model service in the Ark Console. Request id: 021749718307793fc1441381f8ed755824bb9d58a45ee6cb04068', 'param': '', 'type': 'NotFound'}}


curl -X POST https://ark.cn-beijing.volces.com/api/v3/contents/generations/tasks \
  -H "Content-Type: application/json" \
  -H "Authorization: Bearer 21289782-f54f-481b-857e-ba1c8b94927b" \
  -d '{
    "model": "doubao-seaweed-241128",
    "content": [
        {
            "type": "text",
            "text": "女孩抱着狐狸，女孩睁开眼，温柔地看向镜头，狐狸友善地抱着，镜头缓缓拉出，女孩的头发被风吹动  --resolution 720p --duration 5"
        },
        {
            "type": "image_url",
            "image_url": {
                "url": "https://s3.ffire.cc/cdn/20250613/vS2jaDCmA8crncXMR4fB7z_.png"
            }
        }
    ]
}'

curl -X POST https://ark.cn-beijing.volces.com/api/v3/contents/generations/tasks \
  -H "Content-Type: application/json" \
  -H "Authorization: Bearer 05702f67-53ba-438a-a22e-6e6a690b5843" \
  -d '{
    "model": "doubao-seedance-1-0-pro-250528",
    "content": [
        {
            "type": "text",
            "text": "女孩抱着狐狸，女孩睁开眼，温柔地看向镜头，狐狸友善地抱着，镜头缓缓拉出，女孩的头发被风吹动  --resolution 480p --duration 5"
        },
        {
            "type": "image_url",
            "image_url": {
                "url": "https://s3.ffire.cc/cdn/20250613/vS2jaDCmA8crncXMR4fB7z_.png"
            }
        }
    ]
}'

curl -X POST https://ark.cn-beijing.volces.com/api/v3/contents/generations/tasks \
  -H "Content-Type: application/json" \
  -H "Authorization: Bearer e32b1693-147e-40db-a83a-82f85cfa6360" \
  -d '{
    "model": "doubao-seaweed-241128",
    "content": "a dog"
}'


{'id': 'cgt-20250613160030-2dvd7',
 'model': 'doubao-seedance-1-0-pro-250528',
 'status': 'succeeded',
 'content': {'video_url': 'https://ark-content-generation-cn-beijing.tos-cn-beijing.volces.com/doubao-seedance-1-0-pro/02174980163157800000000000000000000ffffac182c17b26890.mp4?X-Tos-Algorithm=TOS4-HMAC-SHA256&X-Tos-Credential=AKLTYjg3ZjNlOGM0YzQyNGE1MmI2MDFiOTM3Y2IwMTY3OTE%2F20250613%2Fcn-beijing%2Ftos%2Frequest&X-Tos-Date=20250613T080120Z&X-Tos-Expires=86400&X-Tos-Signature=5e0928f738f49b93f54923549de4c65940c5007d5e86cb5ebadc756cca3aa03e&X-Tos-SignedHeaders=host'},
 'usage': {'completion_tokens': 246840, 'total_tokens': 246840},
 'created_at': 1749801631,
 'updated_at': 1749801680}
 
 
"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : seconds
# @Time         : 2025/10/31 19:15
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

data = {
    "minimax-hailuo-2.3_768p": 0.22,
    "minimax-hailuo-2.3_1080p": 0.32,
    "minimax-hailuo-2.3-fast_768p": 0.128,
    "minimax-hailuo-2.3-fast_1080p": 0.22,
    "viduq2-turbo_720p": 0.2,
    "viduq2-turbo_1080p": 0.8,
    "viduq2-pro_720p": 0.2,
    "viduq2-pro_1080p": 0.8,

    "doubao-seedance-1-0-pro-fast-251015_480p": 0.025,
    "doubao-seedance-1-0-pro-fast-251015_720p": 0.05,
    "doubao-seedance-1-0-pro-fast-251015_1080p": 0.1
}




if __name__ == '__main__':

     _ = ','.join([k for k, v in data.items() if k.startswith("doubao")])

     print(_)



from pydantic import BaseSettings as _BaseSettings

from meutils.pipe import *


class BaseSetting(_BaseSettings):
    class Config:
        env_file = os.getenv("EVN_FILE")
        env_file_encoding = 'utf-8'


if __name__ == '__main__':
    print(BaseSetting())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : check_api_key
# @Time         : 2024/6/17 08:42
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *


@alru_cache()
async def check(api_key_or_token, check_url):
    headers = {
        "Authorization": f"Bearer {api_key_or_token}",
        "Accept": "application/json"
    }
    payload = {
        "token": api_key_or_token
    }
    try:
        async with httpx.AsyncClient(headers=headers, timeout=60) as client:

            if api_key_or_token.startswith("ey"):

                response: httpx.Response = await client.post(check_url, json=payload)  # free-api: POST /token/check
            else:
                response: httpx.Response = await client.get(check_url)

            # todo: 可扩展

            logger.debug(response.text)
            logger.debug(response.status_code)

            return (
                    response.is_success
                    and response.json().get("is_available")  # deepseek
                    or response.json().get("status", False)  # moonshot、siliconflow
                    or response.json().get("live", False)  # free-api: POST /token/check
            )

    except httpx.RequestError as exc:
        logger.error(exc)
        return False


async def check_api_key_or_token(
        api_keys: Union[str, List[str]],
        check_url: Optional[str] = "https://api.deepseek.com/user/balance",
        return_api_keys: bool = True
):
    """
        {
        "is_available": false,
        "balance_infos": [
            {
                "currency": "CNY",
                "total_balance": "-0.00",
                "granted_balance": "0.00",
                "topped_up_balance": "-0.00"
            }
        ]
        }
    """
    if isinstance(api_keys, str):
        api_keys = [api_keys]

    if not check_url: return api_keys

    tasks = map(partial(check, check_url=check_url), tqdm(api_keys))
    responses = await asyncio.gather(*tasks, return_exceptions=True)
    # responses = await asyncio.gather(*tasks)
    if return_api_keys:
        responses = [k for k, v in zip(api_keys, responses) if v is True]

    return responses




if __name__ == '__main__':
    from meutils.config_utils.lark_utils import get_spreadsheet_values, get_next_token_for_polling

    #
    # api_keys = get_spreadsheet_values(feishu_url="https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=lVghgx", to_dataframe=True)[0].tolist()
    # print(arun(check_deepseek_api_key(api_keys)))

    # api_keys = os.getenv("DEEPSEEK_API_KEY")
    #
    # # arun(check_api_key_or_token(api_keys))
    #
    # # print(arun(check_deepseek_api_key(api_keys)))
    # # print(arun(check('sk-f42b2fdc036247e79cd471ab63b1142d', "https://api.deepseek.com/user/balance")))
    #
    # api_keys = os.getenv("SILICONFLOW_API_KEY").split()
    # for api_key in api_keys:
    #     arun(check(api_key, "https://api.siliconflow.cn/v1/user/info"))

    # api_keys = get_spreadsheet_values(feishu_url="https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=EOZuBW", to_dataframe=True)[0].tolist()

    # kimi
    api_keys = get_spreadsheet_values(feishu_url="https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=Y7HVfo", to_dataframe=True)[0].tolist()
    l = []
    for api_key in api_keys:
        if api_key:
            if arun(check(api_key, "https://all.chatfire.cn/kimi/token/check")):
                l.append(api_key)

    print('\n'.join(l))


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : videos
# @Time         : 2025/10/17 22:55
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.db.redis_db import redis_aclient
from meutils.schemas.video_types import SoraVideoRequest, Video
from meutils.apis.volcengine_apis import videos as volc_videos
from meutils.apis.runware import videos as runware_videos # todo 兼容


class OpenAIVideos(object):

    def __init__(self, api_key: Optional[str] = None, base_url: Optional[str] = None):
        self.api_key = api_key
        self.base_url = base_url

    async def create(self, request: SoraVideoRequest):
        response = {}
        if request.model.startswith("doubao-seedance"):
            response = await volc_videos.create_task(request, self.api_key)  # {'id': 'cgt-20250611152553-r46ql'}

        if task_id := (response.get("id") or response.get("task_id")):
            if self.api_key:
                await redis_aclient.set(task_id, self.api_key, ex=7 * 34 * 3600)

            return Video(id=task_id)

    async def get(self, task_id):
        video = Video(id=task_id)
        if api_key := await redis_aclient.get(task_id):
            api_key = api_key.decode()
        else:
            raise ValueError(f"task_id not found")

        if response := await volc_videos.get_task(task_id, api_key):
            logger.debug(bjson(response))

            status = response.get("status")
            video = Video(id=task_id, status=status, metadata=response)
            if video.status == "completed":
                video.progress = 100
                video.video_url = response.get("content", {}).get("video_url")  # 多个是否兼容

        return video


if __name__ == '__main__':
    model = "doubao-seedance-1-0-pro-fast-251015"
    request = SoraVideoRequest(
        # model=model,
        model=f"{model}_480p",
        # model=f"{model}_720p",
        # model=f"{model}_1080p",

        seconds="4",
        size="720x1280",
    )
    api_key = "267a3b8a-ef06-4d8f-bd24-150f99bb17c1"
    videos = OpenAIVideos(api_key=api_key)

    # video = arun(videos.create(request))

    # Video(id='cgt-20251031183121-zrt26', completed_at=None, created_at=1761906681, error=None, expires_at=None,
    #       model=None, object='video', progress=0, remixed_from_video_id=None, seconds=None, size=None, status='queued',
    #       video_url=None, metadata=None)

    task_id = "cgt-20251031184204-hqcn5"
    arun(videos.get(task_id))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : utils
# @Time         : 2023/11/28 16:51
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

from starlette.requests import Request
from fastapi.security import HTTPBearer, HTTPAuthorizationCredentials

from fastapi import APIRouter, File, UploadFile, Query, Form, BackgroundTasks, Depends, HTTPException as _HTTPException, \
    Request, status


class HTTPException(_HTTPException):
    message: str = ""
    type: str = "error"
    param: Optional[Any] = None
    code: Optional[Any] = None

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        self.detail = self.detail or {
            "error": {
                "message": self.message,
                "type": self.type,
                "param": self.param,
                "code": self.code or self.status_code,
            }
        }


def get_ipaddr(request: Request) -> str:
    """
    Returns the ip address for the current request (or 127.0.0.1 if none found)
     based on the X-Forwarded-For headers.
     Note that a more robust method for determining IP address of the client is
     provided by uvicorn's ProxyHeadersMiddleware.
    """
    if "X_FORWARDED_FOR" in request.headers:
        return request.headers["X_FORWARDED_FOR"]
    else:
        if not request.client or not request.client.host:
            return "127.0.0.1"

        return request.client.host


def get_remote_address(request: Request) -> str:
    """
    Returns the ip address for the current request (or 127.0.0.1 if none found)
    """
    if not request.client or not request.client.host:
        return "127.0.0.1"

    return request.client.host


def limit(limit_value='3/second', error_message: Optional[str] = None, **kwargs):
    """
        @limit(limit_value='3/minute')
        def f(request: Request):
            return {'1': '11'}
    :return:
    """
    from slowapi.errors import RateLimitExceeded
    from slowapi import Limiter, _rate_limit_exceeded_handler
    from slowapi.util import get_remote_address

    limiter = Limiter(key_func=get_remote_address)
    return limiter.limit(limit_value=limit_value, error_message=error_message, **kwargs)


def check_api_key(auth: HTTPAuthorizationCredentials):
    api_key = auth
    if api_key is None:
        detail = {
            "error": {
                "message": "invalid_api_key",
                "type": "invalid_request_error",
                "param": None,
                "code": None,
            }
        }
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail=detail)


if __name__ == '__main__':
    d = {
        "error": {
            "message": "当前分组 chatfire 下对于模型 ERNIE-Bot 无可用渠道 (request id: 20240314180638675032749TVFiqyF1)",
            "type": "new_api_error"
        }
    }



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : common
# @Time         : 2025/6/10 09:11
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 


import math


def calculate_min_resolution(w, h):
    """
    计算给定宽高比的最小像素公约数分辨率（宽高互质）

    参数:
        aspect_ratio (str): 宽高比字符串，例如"16:9"

    返回:
        tuple: (宽, 高) 的元组，整数类型
    """
    # 分割字符串并转换为整数
    w, h = map(int, (w, h))

    # 计算最大公约数
    gcd_val = math.gcd(w, h)

    # 化简为互质的整数比
    width = w // gcd_val
    height = h // gcd_val

    return width, height


def size2aspect_ratio(size):
    if not size: return "1:1"

    if 'x' in size:
        w, h = size.split('x')
        w, h = calculate_min_resolution(w, h)
        return f"{w}:{h}"  # aspect_ratio

    elif ':' in size:
        return size


def size2resolution(size) -> str:
    """
    把任意分辨率换算成最接近的 *p 格式。
    返回 '720p' / '1080p' / '1440p' / '2160p' / '4320p' / '?p'
    """
    w, h = map(int, size.split("x"))

    # 只需看高度；常见档位
    standards = (480, 512, 720, 1080, 1440, 2160, 4320)
    # 找到与 h 最接近的那个
    closest = min(standards, key=lambda x: abs(x - h))
    # 允许 10% 误差，否则认为非常规
    if abs(closest - h) / closest > 0.15:
        return f'{h}p'  # 非标准，直接返回原始高度
    return f'{closest}p'


if __name__ == '__main__':
    print(size2aspect_ratio("1920x1080"))
    print(size2aspect_ratio("1920:1080"))
    print(size2aspect_ratio("1024x1024"))
    print(size2aspect_ratio("16:9"))

    print(size2resolution("512x512"))
    print(size2resolution("480x480"))
    print(size2resolution("1674x1238"))
    print(size2resolution("960x528"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : model_group_info
# @Time         : 2024/11/21 15:52
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : todo 表格

from meutils.pipe import *
from meutils.schemas.oneapi import icons

model_group_info = {
    "Chatfire": {
        "name": "Chatfire",
        "desc": "ChatfireAPI，国内领先的API厂商",
        "icon": "https://registry.npmmirror.com/@lobehub/assets-emoji-anim/1.0.0/files/assets/cowboy-hat-face.webp",
        "notice": "<h1 align = \"center\">[点击查看接入文档](https://api.chatfire.cn/docs)</h1>"
    },
    "OpenAI": {
        "name": "OpenAI",
        "desc": "openai",
        "icon": icons.openai,
        "notice": "<h1 align = \"center\">[点击查看接入文档](https://api.chatfire.cn/docs)</h1>"
    },
    "OpenAI Plus": {
        "name": "OpenAI Plus",

        "desc": "这类模型都是由 Openai ChatGPT逆向工程而来的",
        "icon": icons.openai_plus,
        "notice": "<h1 align = \"center\">[点击查看接入文档](https://api.chatfire.cn/docs)</h1>"
    },

    "Claude": {
        "name": "Claude",
        "desc": "claude",
        "icon": icons.claude,
        "notice": "<h1 align = \"center\">[点击查看接入文档](https://api.chatfire.cn/docs)</h1>"
    },

    "Flux": {
        "name": "Flux",
        "desc": "flux",
        "icon": icons.flux,
        "notice": "<h1 align = \"center\">[点击查看接入文档](https://api.chatfire.cn/docs)</h1>"
    }
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : hivisionidphotos
# @Time         : 2024/9/2 16:52
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

# {
#     '一寸': '一寸',
#     '二寸': '二寸',
#     '小一寸': '小一寸',
#     '小二寸': '小二寸',
#     '大一寸': '大一寸',
#     '大二寸': '大二寸',
#     '五寸': '五寸',
#     '教师资格证': '教师资格证',
#     '国家公务员考试': '国家公务员考试',
#     '初级会计考试': '初级会计考试',
#     '英语四六级考试': '英语四六级考试',
#     '计算机等级考试': '计算机等级考试',
#     '研究生考试': '研究生考试',
#     '社保卡': '社保卡',
#     '电子驾驶证': '电子驾驶证'
# }


class DetailedPhotoSpec(BaseModel):
    url: str

    size: Literal['一寸', '二寸', '小一寸', '小二寸', '大一寸', '大二寸', '五寸', '教师资格证', '国家公务员考试', '初级会计考试', '英语四六级考试', '计算机等级考试', '研究生考试', '社保卡', '电子驾驶证'] = "证件照规格"

    color: Literal['蓝色', '白色', '红色', '自定义底色'] = '蓝色'

    style: Literal['纯色', '上下渐变(白)', '中心渐变(白)'] = '纯色'

    dpi: int = 50  # 10-1000,	# int | float (numeric value between 10 and 1000) in 'KB大小' Slider component

    id_photo_params: list = []

    def __init__(self, /, **data: Any):
        super().__init__(**data)

        self.id_photo_params = [self.url, "尺寸列表", "英语四六级考试", "蓝色", "纯色", "不设置", 0, 0, 0, 413, 295, 10]

        if self.size == '尺寸列表':
            pass

        if self.id_photo_params[5] == '不设置':
            pass


async def create_request(
        url,
        request,
        token: str = None,
        headers: Optional[dict] = None,
        post_kv: Optional[dict] = None
):
    token = token or 'This is a token'
    payload = request if isinstance(request, dict) else request.model_dump(exclude_none=True)

    headers = {
        'Authorization': f'Bearer {token}',
        'Cookie': token,
        **(headers or {})
    }
    async with httpx.AsyncClient(headers=headers, timeout=60) as client:
        response = await client.post(url=url, json=payload, **(post_kv or {}))
        if response.is_success:
            return response.json()
        response.raise_for_status()


if __name__ == '__main__':
    DetailedPhotoSpec(type='一寸')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : files
# @Time         : 2025/1/3 15:38
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :


from meutils.pipe import *
from meutils.io.openai_files import file_extract, guess_mime_type
from meutils.str_utils.json_utils import json_path
from meutils.str_utils.regular_expression import parse_url

from meutils.llm.clients import AsyncOpenAI, zhipuai_client
from meutils.llm.openai_utils import to_openai_params

from meutils.schemas.openai_types import chat_completion, chat_completion_chunk, CompletionRequest, CompletionUsage


class Completions(object):

    def __init__(self, api_key: Optional[str] = None):
        self.api_key = api_key

    async def create(self, request: CompletionRequest):
        logger.debug(request.last_user_content)
        logger.debug(parse_url(request.last_assistant_content))

        if request.last_user_content.startswith("http"):  # 文件问答-单轮
            file_url, *texts = request.last_user_content.split(maxsplit=1) + ["总结下"]
            text = texts[0]

            file_content = await file_extract(file_url, enable_reader=False)

            request.messages = [
                {
                    'role': 'user',
                    'content': f"""{json.dumps(file_content, ensure_ascii=False)}\n\n{text}"""
                }
            ]
        elif image_urls := request.last_urls.get("image_url"):  # 长度为1
            url = image_urls[0]

            if guess_mime_type(url).startswith("image"):  # 图片问答

                if not any(i in request.model for i in {"image", "vl", "vision"}):  ##
                    request.model = "glm-4v-flash"

                for i, message in enumerate(request.messages):
                    if message.get("role") == "user":
                        user_contents = message.get("content")
                        if isinstance(user_contents, list):
                            for i, content in enumerate(user_contents):
                                content_type = content.get("type")

                                if content_type == "image_url":
                                    image_url = content.get("image_url", "")
                                    if isinstance(image_url, dict):
                                        image_url = image_url.get("url", "")

                                    user_contents[i] = {"type": "image_url", "image_url": {"url": image_url}}

            else:  # 文件问答-多轮
                for i, message in enumerate(request.messages[::-1], 1):
                    if message.get("role") == "user":
                        texts = json_path(message, expr='$..text') or [""]
                        file_urls = json_path(message, expr='$..image_url') or [""]

                        # logger.debug(f"""{texts} \n\n {file_urls}""")

                        text, file_url = texts[-1], file_urls[-1]
                        if file_url in image_urls:
                            file_content = await file_extract(file_url, enable_reader=False)

                            message["content"] = f"""{json.dumps(file_content, ensure_ascii=False)}\n\n{text}"""

                            request.messages = request.messages[-i:]
                            break  # 截断：从最新的文件开始

        elif image_urls := parse_url(request.last_assistant_content):
            image_url = image_urls[-1]
            request.messages[-1]["content"] = [
                {"type": "text", "text": request.last_user_content},
                {"type": "image_url", "image_url": {"url": image_url}}
            ]

        logger.debug(request.model_dump_json(indent=4))

        data = to_openai_params(request)

        if request.model.startswith("glm-4v-flash"):
            return await zhipuai_client.chat.completions.create(**data)

        return await AsyncOpenAI(api_key=self.api_key).chat.completions.create(**data)


# data: {"event": "message", "task_id": "900bbd43-dc0b-4383-a372-aa6e6c414227", "id": "663c5084-a254-4040-8ad3-51f2a3c1a77c", "answer": "Hi", "created_at": 1705398420}\n\n
if __name__ == '__main__':
    c = Completions()

    request = CompletionRequest(
        # model="qwen-turbo-2024-11-01",
        # model="claude-3-5-sonnet-20241022",
        model="deepseek-chat",

        messages=[
            # {
            #     'role': 'system',
            #     'content': '你是一个文件问答助手'
            # },
            # {
            #     'role': 'user',
            #     # 'content': {
            #     #     "type": "file_url",
            #     #     "file_url": {"url": "https://oss.ffire.cc/files/招标文件备案表（第二次）.pdf", "detai": "auto"}
            #     # },
            #     'content': [
            #         {
            #             "type": "text",
            #             "text": "这个文件讲了什么？"
            #         },
            #         # 多轮的时候要剔除
            #         {
            #             "type": "file_url",
            #             "file_url": {"url": "https://oss.ffire.cc/files/招标文件备案表（第二次）.pdf", "detai": "auto"}
            #         }
            #     ]
            # },

            {
                'role': 'user',
                # "content": '你好',
                # "content": [
                #     {"type": "text", "text": "https://oss.ffire.cc/files/kling_watermark.png 描述第一张图片"},
                #
                #     # {"type": "text", "text": "描述第一张图片"},
                #     #
                #     # {"type": "image_url", "image_url": "https://oss.ffire.cc/files/kling_watermark.png"},
                #     #     # {"type": "image_url", "image_url": "https://oss.ffire.cc/files/nsfw.jpg"}
                #     #
                # ],

                # 'content': {
                #     "type": "file_url",
                #     "file_url": {"url": "https://mj101-1317487292.cos.ap-shanghai.myqcloud.com/ai/test.pdf", "detai": "auto"}
                # },
                # 'content': "https://oss.ffire.cc/files/%E6%8B%9B%E6%A0%87%E6%96%87%E4%BB%B6%E5%A4%87%E6%A1%88%E8%A1%A8%EF%BC%88%E7%AC%AC%E4%BA%8C%E6%AC%A1%EF%BC%89.pdf 这个文件讲了什么？",
                # 'content': "https://translate.google.com/?sl=zh-CN&tl=en&text=%E6%8F%90%E4%BE%9B%E6%96%B9&op=tr1anslate 这个文件讲了什么？",

                # "content": "https://oss.ffire.cc/files/百炼系列手机产品介绍.docx 总结下"
                # "content": "https://mj101-1317487292.cos.ap-shanghai.myqcloud.com/ai/test.pdf\n\n总结下"

                "content": "http://admin.ilovechatgpt.top/file/4docx_86529298.docx 我无法确定你是否准确识别word里面的论文？",
                # "content": "http://admin.ilovechatgpt.top/file/xinjianMicrosoftWordwendangdoc-9052714901036-bGSJLeKbqQdnIZZn.doc 111111234234",

            },
            #
            # {'role': 'assistant', 'content': "好的"},
            #
            # {
            #     'role': 'user',
            #     # "content": '描述第一张图片',
            #     "content": [
            #         {"type": "text", "text": "描述第一张图片"},
            #
            #         {"type": "image_url", "image_url": "https://oss.ffire.cc/files/nsfw.jpg"}
            #
            #     ],
            #
            # },
            # {
            #     'role': 'user',
            #     'content': [
            #         {
            #             "type": "text",
            #             "text": "总结下"
            #         },
            #
            #         {
            #             "type": "image_url",
            #             "image_url": "xx"
            #         }
            #     ]
            # },
            # {
            #     'role': 'user',
            #     'content': [
            #         {
            #             "type": "text",
            #             "text": "总结下"
            #         },
            #
            #         {
            #             "type": "image_url",
            #             "image_url": "https://oss.ffire.cc/files/招标文件备案表（第二次）.pdf"
            #         }
            #     ]
            # },
            # {
            #     'role': 'user',
            #     'content': [
            #         {
            #             "type": "text",
            #             "text": "总结下"
            #         },
            #
            #         {
            #             "type": "image_url",
            #             "image_url": "https://oss.ffire.cc/files/招标文件备案表（第二次）.pdf"
            #         }
            #     ]
            # },
            # {
            #     'role': 'assistant',
            #     'content': "你好"
            # },
            # {
            #     'role': 'user',
            #     'content': [
            #         {
            #             "type": "text",
            #             "text": "总结下"
            #         },
            #
            #         {
            #             "type": "image_url",
            #             "image_url": "https://oss.ffire.cc/files/招标文件备案表（第二次）.pdf"
            #         }
            #     ]
            # },
            # {
            #     'role': 'assistant',
            #     'content': "1"
            # },
            # {
            #     'role': 'user',
            #     'content': [
            #         {
            #             "type": "text",
            #             "text": "总结1"
            #         }
            #     ]
            # },
            # {
            #     'role': 'assistant',
            #     'content': "2"
            # },
            # {
            #     'role': 'user',
            #     'content': "总结2"
            # },
            #     'content': [
            #         {
            #             "type": "text",
            #             "text": "错了 继续回答"
            #         },
            #         # {
            #         #     "type": "file_url",
            #         #     "file_url": {"url": "https://oss.ffire.cc/files/招标文件备案表（第二次）.pdf", "detai": "auto"}
            #         # }
            #     ]
            # }
        ]

    )

    # [{'role': 'system',
    #   'content': 'undefined\n Current date: 2025-03-13'},
    #  {'role': 'user',
    #   'content': [{'type': 'text', 'text': '解读'},
    #               {'type': 'image_url',
    #                'image_url': 'https://ai.chatfire.cn/files/images/xx-1741850404348-339a12ba3.png'}]},
    #  {},
    #  {'role': 'user', 'content': '总结一下'}]

    request = {
        "model": "gemini-all",
        "messages": [
            {
                "role": "system",
                "content": "\\n Current date: 2025-05-21"
            },
            {
                "role": "user",
                "content": "http://admin.ilovechatgpt.top/file/ceshiwendangdocx_31118702.docx 你好"
            }
        ],
        "stream": True,
        "top_p": 0.7,
        "temperature": 0.8,
        "n": 1
    }
    request = CompletionRequest(**request)
    arun(c.create(request))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : batch_types
# @Time         : 2024/6/3 09:06
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://github.com/datastax/astra-assistants-api/blob/f10ce1e71d321aee2069948758600b88b30b33e0/openapi_server_v2/models/batch.py#L32

from meutils.pipe import *
import pprint

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional, Self


class BatchErrorsDataInner(BaseModel):
    """
    BatchErrorsDataInner
    """  # noqa: E501
    code: Optional[StrictStr] = Field(default=None, description="An error code identifying the error type.")
    message: Optional[StrictStr] = Field(default=None,
                                         description="A human-readable message providing more details about the error.")
    param: Optional[StrictStr] = Field(default=None,
                                       description="The name of the parameter that caused the error, if applicable.")
    line: Optional[StrictInt] = Field(default=None,
                                      description="The line number of the input file where the error occurred, if applicable.")
    __properties: ClassVar[List[str]] = ["code", "message", "param", "line"]

    model_config = {
        "populate_by_name": True,
        "validate_assignment": True,
        "protected_namespaces": (),
    }

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of BatchErrorsDataInner from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
            },
            exclude_none=True,
        )
        # set to None if param (nullable) is None
        # and model_fields_set contains the field
        if self.param is None and "param" in self.model_fields_set:
            _dict['param'] = None

        # set to None if line (nullable) is None
        # and model_fields_set contains the field
        if self.line is None and "line" in self.model_fields_set:
            _dict['line'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of BatchErrorsDataInner from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "code": obj.get("code"),
            "message": obj.get("message"),
            "param": obj.get("param"),
            "line": obj.get("line")
        })
        return _obj


class BatchErrors(BaseModel):
    """
    BatchErrors
    """  # noqa: E501
    object: Optional[StrictStr] = Field(default=None, description="The object type, which is always `list`.")
    data: Optional[List[BatchErrorsDataInner]] = None
    __properties: ClassVar[List[str]] = ["object", "data"]

    model_config = {
        "populate_by_name": True,
        "validate_assignment": True,
        "protected_namespaces": (),
    }

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of BatchErrors from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
            },
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in data (list)
        _items = []
        if self.data:
            for _item in self.data:
                if _item:
                    _items.append(_item.to_dict())
            _dict['data'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of BatchErrors from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "object": obj.get("object"),
            "data": [BatchErrorsDataInner.from_dict(_item) for _item in obj.get("data")] if obj.get(
                "data") is not None else None
        })
        return _obj


class BatchRequestCounts(BaseModel):
    """
    The request counts for different statuses within the batch.
    """  # noqa: E501
    total: StrictInt = Field(description="Total number of requests in the batch.")
    completed: StrictInt = Field(description="Number of requests that have been completed successfully.")
    failed: StrictInt = Field(description="Number of requests that have failed.")
    __properties: ClassVar[List[str]] = ["total", "completed", "failed"]

    model_config = {
        "populate_by_name": True,
        "validate_assignment": True,
        "protected_namespaces": (),
    }

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of BatchRequestCounts from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
            },
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of BatchRequestCounts from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "total": obj.get("total"),
            "completed": obj.get("completed"),
            "failed": obj.get("failed")
        })
        return _obj


class Batch(BaseModel):
    """
    Batch
    """  # noqa: E501
    id: StrictStr
    object: StrictStr = Field(description="The object type, which is always `batch`.")
    endpoint: StrictStr = Field(description="The OpenAI API endpoint used by the batch.")
    errors: Optional[BatchErrors] = None
    input_file_id: StrictStr = Field(description="The ID of the input file for the batch.")
    completion_window: StrictStr = Field(description="The time frame within which the batch should be processed.")
    status: StrictStr = Field(description="The current status of the batch.")
    output_file_id: Optional[StrictStr] = Field(default=None,
                                                description="The ID of the file containing the outputs of successfully executed requests.")
    error_file_id: Optional[StrictStr] = Field(default=None,
                                               description="The ID of the file containing the outputs of requests with errors.")
    created_at: StrictInt = Field(description="The Unix timestamp (in seconds) for when the batch was created.")
    in_progress_at: Optional[StrictInt] = Field(default=None,
                                                description="The Unix timestamp (in seconds) for when the batch started processing.")
    expires_at: Optional[StrictInt] = Field(default=None,
                                            description="The Unix timestamp (in seconds) for when the batch will expire.")
    finalizing_at: Optional[StrictInt] = Field(default=None,
                                               description="The Unix timestamp (in seconds) for when the batch started finalizing.")
    completed_at: Optional[StrictInt] = Field(default=None,
                                              description="The Unix timestamp (in seconds) for when the batch was completed.")
    failed_at: Optional[StrictInt] = Field(default=None,
                                           description="The Unix timestamp (in seconds) for when the batch failed.")
    expired_at: Optional[StrictInt] = Field(default=None,
                                            description="The Unix timestamp (in seconds) for when the batch expired.")
    cancelling_at: Optional[StrictInt] = Field(default=None,
                                               description="The Unix timestamp (in seconds) for when the batch started cancelling.")
    cancelled_at: Optional[StrictInt] = Field(default=None,
                                              description="The Unix timestamp (in seconds) for when the batch was cancelled.")
    request_counts: Optional[BatchRequestCounts] = None
    metadata: Optional[Dict[str, Any]] = Field(default=None,
                                               description="Set of 16 key-value pairs that can be attached to an object. This can be useful for storing additional information about the object in a structured format. Keys can be a maximum of 64 characters long and values can be a maxium of 512 characters long. ")
    __properties: ClassVar[List[str]] = ["id", "object", "endpoint", "errors", "input_file_id", "completion_window",
                                         "status", "output_file_id", "error_file_id", "created_at", "in_progress_at",
                                         "expires_at", "finalizing_at", "completed_at", "failed_at", "expired_at",
                                         "cancelling_at", "cancelled_at", "request_counts", "metadata"]

    @field_validator('object')
    def object_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('batch'):
            raise ValueError("must be one of enum values ('batch')")
        return value

    @field_validator('status')
    def status_validate_enum(cls, value):
        """Validates the enum"""
        if value not in (
            'validating', 'failed', 'in_progress', 'finalizing', 'completed', 'expired', 'cancelling', 'cancelled'):
            raise ValueError(
                "must be one of enum values ('validating', 'failed', 'in_progress', 'finalizing', 'completed', 'expired', 'cancelling', 'cancelled')")
        return value

    model_config = {
        "populate_by_name": True,
        "validate_assignment": True,
        "protected_namespaces": (),
    }

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of Batch from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
            },
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of errors
        if self.errors:
            _dict['errors'] = self.errors.to_dict()
        # override the default output from pydantic by calling `to_dict()` of request_counts
        if self.request_counts:
            _dict['request_counts'] = self.request_counts.to_dict()
        # set to None if metadata (nullable) is None
        # and model_fields_set contains the field
        if self.metadata is None and "metadata" in self.model_fields_set:
            _dict['metadata'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of Batch from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "object": obj.get("object"),
            "endpoint": obj.get("endpoint"),
            "errors": BatchErrors.from_dict(obj.get("errors")) if obj.get("errors") is not None else None,
            "input_file_id": obj.get("input_file_id"),
            "completion_window": obj.get("completion_window"),
            "status": obj.get("status"),
            "output_file_id": obj.get("output_file_id"),
            "error_file_id": obj.get("error_file_id"),
            "created_at": obj.get("created_at"),
            "in_progress_at": obj.get("in_progress_at"),
            "expires_at": obj.get("expires_at"),
            "finalizing_at": obj.get("finalizing_at"),
            "completed_at": obj.get("completed_at"),
            "failed_at": obj.get("failed_at"),
            "expired_at": obj.get("expired_at"),
            "cancelling_at": obj.get("cancelling_at"),
            "cancelled_at": obj.get("cancelled_at"),
            "request_counts": BatchRequestCounts.from_dict(obj.get("request_counts")) if obj.get(
                "request_counts") is not None else None,
            "metadata": obj.get("metadata")
        })
        return _obj


class ListBatchesResponse(BaseModel):
    """
    ListBatchesResponse
    """  # noqa: E501
    data: List[Batch]
    first_id: Optional[StrictStr] = None
    last_id: Optional[StrictStr] = None
    has_more: StrictBool
    object: StrictStr
    __properties: ClassVar[List[str]] = ["data", "first_id", "last_id", "has_more", "object"]

    @field_validator('object')
    def object_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('list'):
            raise ValueError("must be one of enum values ('list')")
        return value

    model_config = {
        "populate_by_name": True,
        "validate_assignment": True,
        "protected_namespaces": (),
    }

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ListBatchesResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
            },
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in data (list)
        _items = []
        if self.data:
            for _item in self.data:
                if _item:
                    _items.append(_item.to_dict())
            _dict['data'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of ListBatchesResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "data": [Batch.from_dict(_item) for _item in obj.get("data")] if obj.get("data") is not None else None,
            "first_id": obj.get("first_id"),
            "last_id": obj.get("last_id"),
            "has_more": obj.get("has_more"),
            "object": obj.get("object")
        })
        return _obj



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : tts
# @Time         : 2023/11/3 15:21
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 增加固定时长的生成

from meutils.pipe import *

import edge_tts


@lru_cache()
def voicesmanager_cls():
    return asyncio.run(edge_tts.VoicesManager.create())


def find_voices(**kwargs: Any):
    """
        find_voices(**{"Locale": "en-US"})
    """
    return voicesmanager_cls().find(**kwargs or {"Locale": "zh-CN"})


# @lru_cache()
async def tts(text, role: str = '云希', filename: Optional[None] = None):
    # https://speech.platform.bing.com/consumer/speech/synthesize/readaloud/voices/list?trustedclienttoken=6A5AA1D4EAFF4E9FB37E23D68491D6F4
    SUPPORTED_VOICES = {
        "晓晓": "zh-CN-XiaoxiaoNeural",
        "晓伊": "zh-CN-XiaoyiNeural",
        "云健": "zh-CN-YunjianNeural",
        "云希": "zh-CN-YunxiNeural",
        "云夏": "zh-CN-YunxiaNeural",
        "云扬": "zh-CN-YunyangNeural",
        "晓北辽宁": "zh-CN-liaoning-XiaobeiNeural",
        "陕西晓妮": "zh-CN-shaanxi-XiaoniNeural",
        "en-US-AriaNeural": "en-US-AriaNeural",
        "en-US-AnaNeural": "en-US-AnaNeural",
        "en-US-ChristopherNeural": "en-US-ChristopherNeural",
        "en-US-EricNeural": "en-US-EricNeural",
        "en-US-GuyNeural": "en-US-GuyNeural",
        "en-US-JennyNeural": "en-US-JennyNeural",
        "en-US-MichelleNeural": "en-US-MichelleNeural",
        "en-US-RogerNeural": "en-US-RogerNeural",
        "en-US-SteffanNeural": "en-US-SteffanNeural"
    }

    # rate: str = "+0%", 加速
    communicate = edge_tts.Communicate(text or "文本为空", voice=SUPPORTED_VOICES.get(role, '陕西晓妮'))

    filename = filename or f"{str(datetime.datetime.now())[:19]} {text[:5]}.mp3"

    await communicate.save(filename)

    return filename


if __name__ == '__main__':
    import asyncio


    def main(text='不知道'):
        # import nest_asyncio
        # nest_asyncio.apply()

        # asyncio.run(tts(text))
        cls = edge_tts.VoicesManager.create()
        asyncio.run(cls)
        return edge_tts.VoicesManager.find(**{"Locale": "zh-CN"})


    #
    #
    # main(text="不知道" * 66)
    # main(text="不知道" * 66)
    #
    # main("不知道11" * 10)
    # main("不知道22" * 10)

    vs = find_voices(**{"Locale": "en-US"})

    print({v['ShortName']: v['ShortName'] for v in vs})



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : images
# @Time         : 2025/4/18 08:55
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
"""
https://www.volcengine.com/docs/85128/1526761
Seedream 通用3.0文生图模型是基于字节跳动视觉大模型打造的新一代文生图模型，本次升级模型综合能力（图文，结构，美感）均显著提升。V3.0参数量更大，对语义有更好的理解，实体结构也更加自然真实，支持 2048 以下分辨率直出，各类场景下的效果均大幅提升。
https://www.volcengine.com/docs/6791/1384311
"""
import os

from meutils.pipe import *
from meutils.caches import rcache

from meutils.decorators.retry import retrying
from meutils.db.redis_db import redis_aclient
from meutils.config_utils.lark_utils import get_next_token_for_polling
from meutils.schemas.jimeng_types import VideoRequest, ImageRequest

from volcengine.visual.VisualService import VisualService
from fastapi import APIRouter, File, UploadFile, Query, Form, Depends, Request, HTTPException, status, BackgroundTasks

FEISHU = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=OiHxsE"


#  and "Access Denied" not in str(r)

@retrying(max_retries=5, predicate=lambda r: "Concurrent Limit" in str(r))  # 限流
async def create_task(request: Union[ImageRequest, VideoRequest, dict], token: Optional[str] = None):
    """https://www.volcengine.com/docs/6791/1399614"""
    token = token or await get_next_token_for_polling(FEISHU)

    logger.debug(token)

    visual_service = VisualService()

    if token:
        ak, sk = token.split('|')
        visual_service.set_ak(ak)
        visual_service.set_sk(sk)

    # request
    if not isinstance(request, dict):
        request = request.model_dump(exclude_none=True)

    response = visual_service.cv_sync2async_submit_task(request)  # 同步转异步

    """
    {'code': 10000,
 'data': {'task_id': '15106285208671192523'},
 'message': 'Success',
 'request_id': '202505291707517FC0D2B135CEE77BC4A5',
 'status': 10000,
 'time_elapsed': '150.967328ms'}
    """

    logger.debug(response)
    if response.get('code') == 10000:
        await redis_aclient.set(response['data']['task_id'], token, ex=7 * 24 * 3600)
    else:
        raise Exception(response)

    return response


#
# @retrying(max_retries=5, predicate=lambda r: "Concurrent Limit" in str(r))  # 限流
# @rcache(ttl=5)
async def get_task(request: dict):
    task_id = request.get("task_id", "")
    token = await redis_aclient.get(task_id)  # 绑定对应的 token
    token = token and token.decode()
    if not token:
        raise HTTPException(status_code=404, detail="TaskID not found")

    visual_service = VisualService()

    if token:
        ak, sk = token.split('|')
        visual_service.set_ak(ak)
        visual_service.set_sk(sk)

    response = visual_service.cv_get_result(request)  # 同步转异步

    logger.debug(response)

    return response


if __name__ == '__main__':
    token = f"""{os.getenv("VOLC_ACCESSKEY")}|{os.getenv("VOLC_SECRETKEY")}"""
    token = "AKLTOWM5ZTc5ZDFhZWNlNDIzODkwYmZiNjEyNzYwNzE0MTI|T0RCbFpHRTJaRFEyWmpjeE5ERXpNR0ptWlRCaU16WmhPRE0wWVdKa01tTQ=="
    prompt = """
    3D魔童哪吒 c4d 搬砖 很开心， 很快乐， 精神抖擞， 背景是数不清的敖丙虚化 视觉冲击力强 大师构图 色彩鲜艳丰富 吸引人 背景用黄金色艺术字写着“搬砖挣钱” 冷暖色对比
    """

    request = ImageRequest(
        req_key="high_aes_general_v30l_zt2i",
        prompt=prompt,
    )

    # request = VideoRequest(
    #     prompt=prompt
    # )

    # arun(create_task(request, token))
    # arun(create_task(request))

    # request = {
    #     "task_id": "141543714223689974",
    #     "req_key": "high_aes_general_v30l_zt2i"
    # }
    # #
    # arun(get_task(request))

    print(bjson(request))

    # {
    #     "req_key": "high_aes_general_v30l_zt2i",
    #     "prompt": "\n    3D魔童哪吒 c4d 搬砖 很开心， 很快乐， 精神抖擞， 背景是数不清的敖丙虚化 视觉冲击力强 大师构图 色彩鲜艳丰富 吸引人 背景用黄金色艺术字写着“搬砖挣钱” 冷暖色对比\n    ",
    #     "seed": -1,
    #     "width": 512,
    #     "height": 512,
    #     "use_pre_llm": true,
    #     "use_sr": false,
    #     "return_url": true,
    #     "logo_info": null
    # }


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : __init__.py
# @Time         : 2021/2/26 4:19 下午
# @Author       : yuanjie
# @WeChat       : 313303303
# @Software     : PyCharm
# @Description  :
from zipfile import ZipFile

from meutils.pipe import *


def to_compress(df: pd.DataFrame, filename, method='zip', archive_name='data.csv', index=False, **to_csv_kwargs):
    """

    @param df:
    @param method: mode is one of {{'infer', 'gzip', 'bz2', 'zip', 'xz', None}}
    @param archive_name:
    @return:
    """
    to_csv_kwargs['index'] = index
    to_csv_kwargs['compression'] = dict(method=method, archive_name=archive_name)
    df.to_csv(filename, **to_csv_kwargs)


@timer("to_excel4dfs")
def to_excel4dfs(df2name_list, path='filename.xlsx', to_excel_kwargs=None):
    """多个sheet写入数据

    :param df2name_list:
    :param to_excel_kwargs:
    :return:
    """
    if to_excel_kwargs is None:
        to_excel_kwargs = {}

    with pd.ExcelWriter(path) as writer:
        for df, sheet_name in df2name_list:
            df.to_excel(writer, sheet_name, **to_excel_kwargs)


def read2write(input_file, output_file, func, batchsize=10e5):
    dfs = pd.read_csv(input_file, sep='\n', chunksize=batchsize, header=None)  # reader

    for df in tqdm(dfs):
        df = func(df)
        df.to_csv(output_file, header=False, index=False, mode='a')  # writer

    # todo 多进程，合并文件， os.getpid() + 时间戳 确定唯一文件


def zipfiles(files, out_file='file.zip', compression=0):
    """
    0 ZIP_STORED：不进行压缩，直接存储
    8 ZIP_DEFLATED：使用DEFLATE算法进行压缩，压缩率较高
    12 ZIP_BZIP2：使用BZIP2算法进行压缩，在某些情况下比DEFLATE更优秀
    14 ZIP_LZMA：使用LZMA算法进行压缩，压缩率最高
    """
    with ZipFile(out_file, compression=compression, mode='w') as f:
        for file in files:
            f.write(file)

    return out_file


if __name__ == '__main__':
    to_compress(pd.DataFrame([1, 2]), 'demo.zip')

    print(pd.read_csv('demo.zip'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : spider
# @Time         : 2024/1/18 12:09
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 经纬度
import json

import requests

url = "https://api.map.baidu.com"

params = {'qt': 'gc',
          'wd': '南京市雨花台区南京南站',
          'cn': '北京市',
          'ie': 'utf-8',
          'oue': '1',
          'fromproduct': 'jsapi',
          'v': '2.1',
          'res': 'api',
          'callback': 'BMap._rd._cbk57312',
          'ak': 'CG8eakl6UTlEb1OakeWYvofh',
          'seckey': 
              'WDTCqRTPViBQC9Ntv0XlrvG64IBwuCDpqNqu6KxRpWc=,7l5VkLxKddt3oR9nwbL-1orEnLu4akSUBIJtOhRdQkb2ygfQkyF-9TRkDdRZ7S_QC0jX2agOfZKRm8-xer6QNJhpBAbzbUxcF9EoqGsPNM7AIXv0ir0egW1Rn_sopbnwTHImbltrTpXrnW4-0t6P9X43lh8-DlPyZItvM2283GAc4hz7IiJcGYB3n_GLO9AE-9vGy_QGY88XZ8UatiJA8A',
          'timeStamp': '1730434021654',
          'sign': 'd8afe5934cbd'
          }


response = requests.request("GET", url, params=params)

data = response.text.strip(')').split('(', 1)[-1]

json.loads(data)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : X.
# @File         : __init__.py
# @Time         : 2020/11/12 10:54 上午
# @Author       : yuanjie
# @Email        : meutils@qq.com
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *

__version__ = Path(get_resolve_path('./data/VERSION', __file__)).read_text()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : videos
# @Time         : 2025/1/15 15:45
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.apis.fal.images import check

from meutils.schemas.task_types import TaskResponse
from meutils.schemas.video_types import VideoRequest

from meutils.schemas.fal_types import FEISHU_URL
from meutils.config_utils.lark_utils import get_next_token_for_polling

from fal_client import AsyncClient


# 平台/模型/版本
models_mapping ={
    "kling-video/v2/master/image-to-video": "latentsync",
    "kling-video/v2/master/text-to-video": "latentsync",
    "kling-video/lipsync/audio-to-video": "latentsync", # todo
    "kling-video/lipsync/text-to-video": "latentsync",
}

async def create_task(request: VideoRequest, token: Optional[str] = None):
    """https://fal.ai/models/fal-ai/latentsync/api#queue-submit

    """
    token = token or await get_next_token_for_polling(feishu_url=FEISHU_URL, from_redis=True, check_token=check)
    logger.debug(request)

    application = f"fal-ai/{request.model.removeprefix('fal-ai/')}"

    client = AsyncClient(key=token)
    response = await client.submit(
        application=application,
        arguments=request.model_dump(exclude_none=True, exclude={"model"})
    )
    # AsyncRequestHandle(request_id='0b7ab6b8-c7dc-4f17-a655-4ee56dd0f864')

    return TaskResponse(task_id=f"{request.model}::{response.request_id}", system_fingerprint=token)


@alru_cache(ttl=5)
async def get_task(task_id: str, token: Optional[str] = None):
    model, request_id = task_id.rsplit('::', 1)
    application = f"fal-ai/{model.removeprefix('fal-ai/')}"

    client = AsyncClient(key=token)
    response = await client.status(application, request_id, with_logs=True)

    logger.debug(response.__class__.__name__)  # Queued, InProgress, COMPLETED

    if metrics := response.__dict__.get("metrics"):  # {'inference_time': 17.29231595993042}
        response = await client.result(application, request_id)

    return response


if __name__ == '__main__':
    model = "latentsync"
    # model = "sync-lipsync"


    model = "fal-ai/kling-video/v2/master/image-to-video"
    model = "fal-ai/kling-video/v1/standard/text-to-video"

    request = VideoRequest(
        model=model,
        prompt="A cute cat",
    )


    r = arun(create_task(request))


    # r = arun(get_task(task_id, token))

    arun(get_task(r.task_id, r.system_fingerprint))


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : __init__.py
# @Time         : 2024/7/5 12:05
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : todo: # 雷军 "738d0cc1a3e9430a9de2b544a466a7fc" 增加模型映射表
# https://docs.fish.audio/api-reference/endpoint/openapi-v1/speech-to-text
# https://docs.fish.audio/text-to-speech/create-model
import shortuuid
# https://api.fish.audio/文档

from fish_audio_sdk import Session, TTSRequest, ASRRequest

from meutils.pipe import *
from meutils.io.files_utils import to_url_fal as to_url

from meutils.config_utils.lark_utils import get_next_token_for_polling
from meutils.notice.feishu import send_message as _send_message

BASE_URL = "https://api.fish.audio"
FEISHU_URL = "https://xchatllm.feishu.cn/sheets/ZUyLsVyjMhtexRt3IB9ci9fUnWc?sheet=7ce4e3"
FEISHU_URL_VIP = "https://xchatllm.feishu.cn/sheets/ZUyLsVyjMhtexRt3IB9ci9fUnWc?sheet=QLTif4"
FEISHU_URL_API = "https://xchatllm.feishu.cn/sheets/ZUyLsVyjMhtexRt3IB9ci9fUnWc?sheet=v5C5xa"

send_message = partial(
    _send_message,
    title=__name__,
    url="https://open.feishu.cn/open-apis/bot/v2/hook/f0475882-ad39-49d9-ad77-523c0e768e96"
)

COVER_IMAGE = get_resolve_path('../../data/cowboy-hat-face.webp', __file__).read_bytes()


async def check_token(token: str, for_api: bool = False):
    path = "api-credit" if for_api else "package"
    try:
        headers = {
            "Authorization": f"Bearer {token}"
        }
        async with httpx.AsyncClient(base_url=BASE_URL, headers=headers) as client:
            response = await client.get(f"/wallet/self/{path}")
            response.raise_for_status()
            data = response.json()

            # logger.debug(data)

            credit = data.get('credit') or data.get('balance', 0)
            logger.info(credit)

            return float(credit) > 0
    except Exception as e:
        logger.error(e)
        return False


async def get_model_list(token, **kwargs):  # 上传的音色
    params = {
        'self': True,
        'title': None,

        'page_size': 10,
        'page_number': 1,
        **kwargs
    }
    headers = {
        "authorization": f"Bearer {token}"
    }
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers) as client:
        response = await client.get("/model", params=params)
        return response.is_success and response.json()


async def create_tts_model(title, voices: List[bytes], texts: Optional[List[str]] = None):
    session = Session(await get_next_token_for_polling(feishu_url=FEISHU_URL_API))  # 逆向token也兼容

    model = session.create_model(
        title=title,
        visibility="public",

        texts=texts,
        voices=voices,

        cover_image=COVER_IMAGE,
    )
    return model


async def create_tts(
        request: TTSRequest,
        token: Optional[str] = None,
        response_format: Optional[Literal["url",]] = None
):
    if len(request.text.encode()) < 500:
        token = token or await get_next_token_for_polling(feishu_url=FEISHU_URL)
    else:
        token = token or await get_next_token_for_polling(feishu_url=FEISHU_URL_VIP)

    headers = {
        "Authorization": f"Bearer {token}",
    }

    payload = {
        "type": "tts",
        "channel": "free",
        "stream": True,
        "model": request.reference_id or "",  # 雷军 "738d0cc1a3e9430a9de2b544a466a7fc", # todo: 支持下 btyes
        "parameters": {
            "text": request.text
        },
        "format": request.format,
    }
    try:
        async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=300) as client:
            response = await client.post("/task", json=payload)
            response.raise_for_status()

            if response_format == "url":
                # logger.debug(response.headers)

                task_id = response.headers.get("task-id")
                task_response = await client.get(f"/task/{task_id}")
                data = task_response.json()
                data["result"] = await to_url(data["result"], filename=f'{shortuuid.random()}.mp3')  # 转存 翻墙的
                return data
            return response.content  # bytes

    except Exception as e:
        logger.error(e)
        session = Session(await get_next_token_for_polling(feishu_url=FEISHU_URL_API))
        files = session.tts(TTSRequest(**request.model_dump()))

        if response_format == "url":
            url = await to_url(b"".join(files), filename=f'{shortuuid.random()}.mp3')

            return {
                "_id": shortuuid.random(),
                "state": "finished",
                "type": "tts",
                "model": request.reference_id,
                "parameters": {
                    "text": request.text
                },
                "created_at": datetime.datetime.now().isoformat(),
                "deleted": False,
                "channel": "premium",
                "finished_at": datetime.datetime.now().isoformat(),
                "result": url,
                "generate_error": False
            }


# async def create_tts_model(token, voices: list, **kwargs):
#     """
#     {'_id': '9d10cdbea3954aa9b8fd992fd24b92a7',
#      'author': {'_id': 'd71d7c63c52e4d70be72e3afdb7952ab',
#                 'avatar': '',
#                 'nickname': '313303303'},
#      'cover_image': 'coverimage/9d10cdbea3954aa9b8fd992fd24b92a7',
#      'created_at': '2024-07-05T05:30:32.020433Z',
#      'description': '',
#      'languages': ['zh'],
#      'like_count': 0,
#      'liked': False,
#      'mark_count': 0,
#      'marked': False,
#      'samples_text': [],
#      'shared_count': 0,
#      'state': 'trained',
#      'tags': [],
#      'task_count': 0,
#      'title': 'chatfire',
#      'train_mode': 'fast',
#      'type': 'tts',
#      'updated_at': '2024-07-05T05:30:32.020407Z',
#      'visibility': 'public'}
#     """
#     payload = {
#         **kwargs
#     }
#
#     files = [
#         ("cover_image", get_resolve_path('cowboy-hat-face.webp', __file__).read_bytes())
#     ]
#
#     for voice in voices:
#         files.append(("voices", voice))  # bytes
#
#     data = [
#         ("visibility", "private"),
#         ("type", "tts"),
#         ("title", "Demo"),
#         ("train_mode", "fast"),
#         # Enhance audio quality will remove background noise
#         ("enhance_audio_quality", "true"),
#         # Texts are optional, but if you provide them, they must match the number of audio samples
#         ("texts", "text1"),
#         ("texts", "text2"),
#     ],
#
#     files = {
#         'title': (None, 'chatfire-tts'),
#         'description': (None, ''),
#         'type': (None, 'tts'),
#         'train_mode': (None, 'fast'),
#         'visibility': (None, 'public'),  # private
#
#         'voices': file,  # ('audio_name.mp3', file)
#         'cover_image': open(get_resolve_path('cowboy-hat-face.webp', __file__), 'rb')
#
#     }
#
#     headers = {
#         "authorization": f"Bearer {token}"
#     }
#     async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=100) as client:
#         response = await client.post("/model", json=payload, files=files)
#         # logger.debug(response.text)
#         if response.is_success:
#             _ = response.json()
#             _['model_id'] = _['_id']
#             return _
#         else:
#             return response.text


if __name__ == '__main__':
    token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJ1c2VyX2lkIjoiZDcxZDdjNjNjNTJlNGQ3MGJlNzJlM2FmZGI3OTUyYWIiLCJleHAiOjE3MzAyODYwNjguNTMyMTA5NX0.GNeW2WLlBqyP6evNQFkXpqa0UYOg50_IwNO48DuJc04"
    # print(bjson(arun(get_model_list(token))))
    # file = open('/Users/betterme/Downloads/whisper-1719913495729-54f08dde5.wav.mp3.mp3', 'rb')

    # arun(create_tts_model(title="chatfire-tts", voices=[Path("x.mp3").read_bytes()]))

    # model="cf8ea9c2c9e947a6b9d0fccec68c9dbd"

    # token = "5cf445234491429c9855d3fa666d49f6"

    # arun(check_token(token, for_api=True))

    request = TTSRequest(text="语音克隆" * 20, reference_id="cf8ea9c2c9e947a6b9d0fccec68c9dbd")
    arun(create_tts(request, response_format="url"))
    # text = arun(create_tts(request))
    #
    # Path('x.mp3').write_bytes(text)

    # print(get_resolve_path('../../data/cowboy-hat-face.webp', __file__))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : 鉴权
# @Time         : 2024/11/25 10:07
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
import time
import jwt

ak = ""  # 填写access key
sk = ""  # 填写secret key


def encode_jwt_token(ak, sk):
    headers = {
        "alg": "HS256",
        "typ": "JWT"
    }
    payload = {
        "iss": ak,
        "exp": int(time.time()) + 1800,  # 有效时间，此处示例代表当前时间+1800s(30min)
        "nbf": int(time.time()) - 5  # 开始生效的时间，此处示例代表当前时间-5秒
    }
    token = jwt.encode(payload, sk, headers=headers)
    return token


api_token = encode_jwt_token(ak, sk)
print(api_token)  # 打印生成的API_TOKEN



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : ark
# @Time         : 2024/12/31 13:44
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import json
import os

from meutils.pipe import *
import volcenginesdkcore
from volcenginesdkcore.signv4 import SignerV4
import volcenginesdkark

# sign(path, method, headers, body, query, ak, sk, region, service)

headers = {}
payload = {
    "Prompt": "关于星空的歌",
    "Genre": "R&B/Soul",
    "Mood": "Dynamic/Energetic",
    "Gender": "Male",
    "ModelVersion": "v4.0"
}
params = {"Action": "GenLyrics", "Version": "2024-08-12"}
SignerV4.sign(
    path="/",
    method="POST",
    query=params,
    body=json.dumps(payload),
    headers=headers,
    ak=os.getenv("ARK_ACCESS_KEY"),
    sk=os.getenv("ARK_SECRET_ACCESS_KEY"),
    region="cn-beijing",
    service="imagination"
)

print(headers)

s = ''
for k, v in headers.items():
    s += f"{k}:{v}\n"

print(s)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : files
# @Time         : 2024/11/11 13:17
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.io.files_utils import to_url

import fal_client


async def submit():
    handler = await fal_client.submit_async(
        "fal-ai/flux/dev/image-to-image",
        arguments={
            "image_url": "https://fal.media/files/koala/Chls9L2ZnvuipUTEwlnJC.png",
            "prompt": "一只猫带着眼镜"
        },
        # webhook_url="https://optional.webhook.url/for/results",
    )

    request_id = handler.request_id
    print(request_id)


if __name__ == '__main__':
    # 02ec80b4-4bac-42e4-83bb-f4e1c32e77f7

    # arun(submit())

    # fal_client.upload

    with timer():
        pass
        # p = Path("/Users/betterme/PycharmProjects/AI/100.png")
        #
        # print(fal_client.upload_file(p))

        # fal_client.upload()
        import fal_client

        url = fal_client.upload_file(Path("/Users/betterme/PycharmProjects/AI/MeUtils/meutils/ai_video/douyin.mp4"))

        # arun(to_url(Path("/Users/betterme/PycharmProjects/AI/MeUtils/meutils/ai_video/douyin.mp4").read_bytes(), content_type=None))

        import mimetypes

        mimetypes.guess_type('x.wav')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : xx
# @Time         : 2023/11/17 16:12
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
import os
from datetime import timedelta
from scenedetect.scene_manager import SceneManager


# 将时间戳转换为字符串格式
def timestamp_to_string(timestamp):
    return str(timedelta(seconds=timestamp))


# 获取场景列表
scene_list = SceneManager.get_scene_list('scene_list.csv')

# 遍历输出的图像文件并重命名
for i, filename in enumerate(os.listdir('.')):
    if filename.startswith('scene') and filename.endswith('.png'):
        timestamp = scene_list[i].get_timecode().get_seconds()
        new_filename = timestamp_to_string(timestamp).replace(':', '-') + '.png'
        os.rename(filename, new_filename)



import cv2


def get_video_duration(video_path):
    cap = cv2.VideoCapture(video_path)
    frame_count = int(cap.get(cv2.CAP_PROP_FRAME_COUNT))
    frame_rate = int(cap.get(cv2.CAP_PROP_FPS))
    duration = frame_count / frame_rate
    return duration


video_path = "example.mp3"
duration = get_video_duration(video_path)
print("视频时长：{}秒".format(duration))


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : cache
# @Time         : 2023/8/24 09:22
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 通用缓存， todo: redis缓存

from meutils.pipe import *
from cachetools import cached, cachedmethod, LRUCache, RRCache, TTLCache
from joblib import hashing  # hashing.hash
from joblib.func_inspect import filter_args

key_fn = lambda *args, **kwargs: hashing.hash((args, kwargs))


def ttl_cache(maxsize=128, ttl=np.inf):
    cache = cached(TTLCache(maxsize, ttl), key=key_fn)

    @wrapt.decorator
    def inner(wrapped, instance, args, kwargs):
        wrapped = cache(wrapped)
        return wrapped(*args, **kwargs)

    return inner


def hash_key(func: Callable, ignore: Optional[List] = None, *args, **kwargs):
    fn_name = func.__name__ if hasattr(func, '__name__') else 'fn'

    raw_key = dict(filter_args(func, [], list(args), kwargs))  # {'x': 1, '**': {'a': 1, 'b': 2, 'c': 3}, '*': [2, 3]}
    raw_key = {**raw_key.pop('**', {}), **raw_key}

    # self cls 看场景选择
    for arg in ignore or []:
        raw_key.pop(arg, None)

    key = hashing.hash(raw_key)

    return f"cache:{fn_name}:{key}"


def redis_cache(maxsize=128, ttl=np.inf, ignore=None, verbose=1, tag=None):
    cache = cached(TTLCache(maxsize, ttl), key=key_fn)

    @wrapt.decorator
    def inner(wrapped, instance, args, kwargs):
        wrapped = cache(wrapped)
        return wrapped(*args, **kwargs)

    return inner


@decorator
def redis_cache(func, rc=None, ttl=3, ignore=None, verbose=1, tag=None, *args, **kwargs):
    """redis 缓存"""
    ##############################################################################
    raw_key = dict(filter_args(func, [], list(args), kwargs))
    raw_key = {**raw_key.pop('**', {}), **raw_key, '__tag__': tag or func.__name__}

    # self cls 看场景选择
    ignore = (ignore or []) + [
        'api_base', 'api_key', 'openai_api_base', 'openai_api_key',
        'organization', 'openai_organization',
        'request_timeout'
    ]
    for arg in ignore:
        raw_key.pop(arg, None)

    key = hashing.hash(raw_key)

    k = f"cache:{func.__name__}:{key}"

    ##############################################################################

    if k in rc:
        return pickle.loads(rc.get(k))
    else:
        if verbose: logger.info(f"CacheKey: {k}")

        _ = func(*args, **kwargs)
        rc.set(k, pickle.dumps(_), ex=ttl)
        return _


if __name__ == '__main__':
    @ttl_cache()
    def f(x):
        time.sleep(1)
        return x

    # with timer(1):
    #     print(f([1, 2]))
    # with timer(1):
    #     print(f([1, 2]))
    #
    # with timer(2):
    #     print(f(range(10)))

    # def fn(x):
    #     time.sleep(1)
    #     return x

    # hash_key(fn, ignore=[], a=1, b=2, c=3)

    # print(filter_args(fn, [], [1, 2, 3], {'a': 1, 'b': 2, 'c': 3}))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : todo
# @Time         : 2023/5/8 13:49
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from urllib.parse import urlparse

url = 'https://www.example.com/path/to/something?a=1&b=2#fragment'
parsed_url = urlparse(url)

print(parsed_url.scheme)  # https
print(parsed_url.netloc)  # www.example.com
print(parsed_url.path)  # /path/to/something
print(parsed_url.query)  # a=1&b=2
print(parsed_url.fragment)  # fragment


@timer('1')
@timer('2')
def f():
    pass



if __name__ == '__main__':
    f()
    print(get_name())


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : minio_utils
# @Time         : 2024/1/3 14:42
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://www.cnblogs.com/fanpiao/p/17603553.html
from minio import Minio
from meutils.pipe import *

minioClient = Minio(
    endpoint=os.getenv('MINIO_ENDPOINT'),
    access_key=os.getenv('MINIO_ACCESS_KEY'),
    secret_key=os.getenv('MINIO_SECRET_KEY'),
    # secure=False,
)

# Make a bucket with the make_bucket API call.
bucket_name = 'bname'
# minioClient.make_bucket(bucket_name)

print(minioClient.list_buckets())
# Put an object 'pumaserver_debug.log' with contents from 'pumaserver_debug.log'.

# minioClient.fput_object(bucket_name, 'object_name', 'x.yml')
#
# minioClient.fput_object(bucket_name, 'img.png', 'img.png')
#
# minioClient.fput_object(bucket_name, 'file.py', 'file.py')


# minioClient.fput_object(bucket_name, 'x/img','img.png', content_type='image/png')


# url = minioClient.get_presigned_url(
#     "GET",
#     bucket_name,
#     'img.png',
# )
# print(url)

# url = minioClient.presigned_get_object(bucket_name, 'img.png', )
# print(url)
#
# response = minioClient.get_object(bucket_name, 'img.png', )
# print(len(response.read()))


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : gitee_types
# @Time         : 2025/7/28 16:09
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=gg5DNy"

BASE_URL = "https://ai.gitee.com/v1"


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : luca
# @Time         : 2024/8/20 10:39
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.config_utils.lark_utils import get_spreadsheet_values, get_next_token_for_polling
from meutils.notice.feishu import send_message as _send_message
from meutils.schemas.task_types import TaskType, Task, FileTask
from meutils.io.files_utils import base64_to_bytes

from fastapi import status, HTTPException

FEISHU_URL = 'https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=P5wJMA'
BASE_URL = "https://luca.cn"

send_message = partial(
    _send_message,
    title=__name__,
    url="https://open.feishu.cn/open-apis/bot/v2/hook/dc1eda96-348e-4cb5-9c7c-2d87d584ca18"
)


async def upload(file: bytes, token: Optional[str] = None):
    token = token or await get_next_token_for_polling(feishu_url=FEISHU_URL)

    files = [('file', ('x.png', file, 'image/png'))]

    headers = {
        'token': token,
    }
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=100) as client:
        response = await client.post("/api/img/v1/uploadImg", files=files)

        logger.debug(response.status_code)
        logger.debug(response.text)

        if response.is_success:
            data = response.json()
            file_id = data['data']['imageId']
            if file_id:
                return FileTask(id=file_id, data=data, system_fingerprint=token)
        raise HTTPException(status_code=status.HTTP_451_UNAVAILABLE_FOR_LEGAL_REASONS, detail="触发内容审核")


async def create_chat(prompt, image_data, token: Optional[str] = None):
    token = token or await get_next_token_for_polling(feishu_url=FEISHU_URL)

    # image_data: base64 url bytes/path
    file = None
    if isinstance(image_data, bytes): # todo: 抽象成通用
        file = image_data

    elif len(image_data) < 512 and Path(image_data).is_file():  # base64
        file = Path(image_data).read_bytes()

    elif image_data.startswith('data:'):  # base64
        file = base64_to_bytes(image_data)

    elif image_data.startswith('http'):
        resp = await httpx.AsyncClient(timeout=100, follow_redirects=True).get(image_data)
        file = resp.content

    file_id = (await upload(file, token)).id

    headers = {
        'token': token,
    }
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=100) as client:
        payload = {"title": "新会话"}
        response = await client.post("/api/chat/v1/createConv", json=payload)

        # logger.debug(response.status_code)
        # logger.debug(response.text)

        conversationId = response.json()['data']

        payload = {
            "generateType": "NORMAL",
            "conversationId": conversationId,
            "parentMessageId": "",
            "chatMessage": [
                {
                    "role": "USER",
                    "contents": [
                        {
                            "type": "IMAGE",
                            "imageId": file_id,
                            "pairs": "",

                        },
                        {
                            "type": "TEXT",
                            "imageId": "",
                            "pairs": prompt,
                        }
                    ],

                    "id": "",
                    "parentMsgId": "",

                    # "contentLayout": "<p><img src=\"blob:https://luca.cn/86f60c82-b283-4f55-8445-36b86381729a\" alt=\"\" data-href=\"Dn6AUbGGW7UXATs86M9rV\" style=\"\"/>总结下</p><p><br></p>",
                }
            ]
        }
        response = await client.post("/api/chat/v1/submitMsg", json=payload)
        data = response.json()
        logger.debug(data)

        messageId = data['data']['childMsgId']
        payload = {
            "conversationId": conversationId,
            "messageId": messageId
        }
        for i in range(10):
            await asyncio.sleep(1)
            response = await client.post('/api/chat/v1/queryMsg', json=payload)
            # logger.debug(response.status_code)
            # logger.debug(response.json())
            # logger.debug(response.json()['data'].get('output'))
            if _ := response.json()['data'].get('output'): return _


if __name__ == '__main__':
    # file = open("/Users/betterme/PycharmProjects/AI/11.jpg", 'rb').read()
    # arun(upload(file))
    # arun(create_chat("图片里有什么", "/Users/betterme/PycharmProjects/AI/11.jpg"))

    url = 'https://dss2.bdstatic.com/5bVYsj_p_tVS5dKfpU_Y_D3/res/r/image/2021-3-4/hao123%20logo.png'
    arun(create_chat("图片里有什么", url))

    # arun(get_next_token_for_polling(feishu_url=FEISHU_URL))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : text_split
# @Time         : 2023/4/23 20:22
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *


# 分句，句子长度累加到 512 组合到一起，重叠n个句子或者size，
def text_split(text, window_size=512, overlap_rate=0.):  # 简单粗暴
    return str(text.strip()).lower() | xgroup(window_size, overlap_rate)


if __name__ == '__main__':
    text = """
    姚明（Yao Ming），男，汉族，无党派人士，1980年9月12日出生于上海市徐汇区，祖籍江苏省苏州市吴江区震泽镇，前中国职业篮球运动员，司职中锋，现任亚洲篮球联合会主席、中国篮球协会主席、中职联公司董事长兼总经理， [1-3]   十三届全国青联副主席， [4]  改革先锋奖章获得者。 [5]  第十四届全国人大代表 [108]  。
1998年4月，姚明入选王非执教的国家队，开始篮球生涯。2001夺得CBA常规赛MVP，2002年夺得CBA总冠军以及总决赛MVP，分别3次当选CBA篮板王以及盖帽王，2次当选CBA扣篮王。在2002年NBA选秀中，他以状元秀身份被NBA的休斯敦火箭队选中，2003-09年连续6个赛季（生涯共8次）入选NBA全明星赛阵容，2次入选NBA最佳阵容二阵，3次入选NBA最佳阵容三阵。2009年，姚明收购上海男篮，成为上海久事大鲨鱼俱乐部老板。2011年7月20日，姚明宣布退役。
2013年，姚明当选为第十二届全国政协委员。2015年2月10日，姚明正式成为北京申办冬季奥林匹克运动会形象大使之一。2016年4月4日，姚明正式入选2016年奈史密斯篮球名人纪念堂，成为首位获此殊荣的中国人；10月，姚明成为中国“火星大使”；11月，当选CBA公司副董事长。 [6]
2017年10月20日，姚明已将上海哔哩哔哩俱乐部全部股权转让。 [7]  2018年9月，荣获第十届“中华慈善奖”慈善楷模奖项。 [8]  2019年10月28日，胡润研究院发布《2019胡润80后白手起家富豪榜》，姚明以22亿元排名第48。
    """.strip()

    for i in text_split(text, 200, 0.1):
        print('='*100)
        print(i)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : images
# @Time         : 2025/10/22 11:10
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import httpx

from meutils.pipe import *
from meutils.caches import rcache
from meutils.llm.clients import AsyncClient
from meutils.decorators.retry import retrying, IgnoredRetryException

from meutils.llm.openai_utils import to_openai_params
from meutils.io.files_utils import to_png, to_url_fal, to_url
from meutils.notice.feishu import send_message_for_images
from meutils.schemas.image_types import ImageRequest, ImagesResponse

base_url = "https://api.freepik.com"

"""FPSXc7a13cdcd4893ff3aa053749d05485a7
 curl --request POST \
  --url https://api.freepik.com/v1/ai/gemini-2-5-flash-image-preview \
  --header 'Content-Type: application/json' \
  --header 'x-freepik-api-key: FPSX3e216c84cc281f6f0f5f605334e22ad0' \
  --data '{
  "prompt": "A cat",
  "webhook_url": "https://openai-dev.chatfire.cn/sys/webhook/freepik"
}'


curl --request GET \
  --url https://api.freepik.com/v1/ai/gemini-2-5-flash-image-preview/0ef796f9-1efe-421a-935b-f1dfb4a3cb32 \
  --header 'x-freepik-api-key: FPSX3e216c84cc281f6f0f5f605334e22ad0'
"""


async def generate(request: ImageRequest, api_key: Optional[str] = None, base_url: Optional[str] = None):
    task_id = await create_task(request, api_key)

    for i in tqdm(range(20)):
        if data := await get_task(task_id, api_key, request):
            return data

        await asyncio.sleep(3)


@retrying(ignored_exception_types=IgnoredRetryException)
async def get_task(task_id, api_key, request: ImageRequest):
    headers = {"x-freepik-api-key": api_key}
    client = AsyncClient(api_key=api_key, base_url=base_url, default_headers=headers)

    response = await client.get(f"/v1/ai/{request.model}/{task_id}", cast_to=object)

    logger.debug(bjson(response))

    if (data := response.get("data")) and (images := data.get("generated")):
        return ImagesResponse(image=images)
    elif data.get("status") == "FAILED":
        raise IgnoredRetryException(data)

    """
    {
    "code": 200,
    "result": {
        "task_id": "ddnxnz9cyhyr67vpd4",
        "user_id": 10061,
        "version": "44b9310748ecdccd1dfa60d68efe35b4a6291453d5edfad417075890d55a208f",
        "error": "The input or output was flagged as sensitive. Please try again with different inputs. (E005) (uIJ6l3ruRD)", #####
        "total_time": 5.0,
        "predict_time": null,
        "logs": null,
        "output": [],
        "status": "failed",
        "create_at": null,
        "completed_at": null
    },
    "message": {}
    }

    """


@rcache(ttl=3600)
async def create_task(request: ImageRequest, api_key: Optional[str] = None):
    headers = {"x-freepik-api-key": api_key}
    payload = {
        "prompt": request.prompt,
        "reference_images": request.image_urls,
    }

    client = AsyncClient(api_key=api_key, base_url=base_url, default_headers=headers)

    response = await client.post(
        f"/v1/ai/{request.model}",
        body=payload,
        cast_to=object
    )

    # async with httpx.AsyncClient(base_url=base_url, headers=headers, timeout=30) as client:
    #     response = await client.post(f"/v1/ai/{request.model}", json=payload)
    #     response.raise_for_status()
    #     response = response.json()

    logger.debug(bjson(response))

    if task_id := (response.get("data") or {}).get("task_id"):
        return task_id

    raise Exception(f"Create Task Failed: {response}")


if __name__ == '__main__':
    api_key = "FPSXc7a13cdcd4893ff3aa053749d05485a7"
    model = "gemini-2-5-flash-image-preview"

    request = ImageRequest(
        model=model,
        prompt="带个墨镜    ",
        image=["https://s3.ffire.cc/files/jimeng.jpg"],
    )

    # arun(create_task(request, api_key))
    #
    # arun(get_task('bb8c92f1-5af6-420b-a6b7-56d8f4b70faa', api_key, request))

    arun(generate(request, api_key))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : kling
# @Time         : 2024/11/28 16:18
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.decorators.retry import retrying
from meutils.async_utils import async_to_sync_pro
from meutils.async_task import worker, shared_task


class Request(BaseModel):
    method: str = "GET"
    url: str = "https://api.chatfire.cn/"

    class Config:
        frozen = True


@shared_task
def do_sync_task(sleep=10, **kwargs):
    logger.debug("同步任务")
    time.sleep(sleep)

    return kwargs


# AttributeError: 'AsyncToSync' object has no attribute '__name__'. Did you mean: '__ne__'?


@alru_cache()
async def create_task(request: Request, **kwargs):
    logger.debug(request)
    await asyncio.sleep(10)
    return request


worker.task()


@shared_task(pydantic=True)
@async_to_sync_pro
async def ado_pydantic_task(request: Request):
    logger.debug("同步任务+协程+结构体+缓存")
    logger.debug(request)
    if isinstance(request, dict):
        request = Request(**request)

    return await create_task(request)


@worker.task(pydantic=True)
@async_to_sync_pro
async def ado_pydantic_task_2(request: Request):
    logger.debug("同步任务+协程+结构体+缓存")
    logger.debug(request)

    return await create_task(request)


@shared_task(pydantic=True)
def do_pydantic_task(request: Request) -> Request:
    logger.debug(request)
    return request


@shared_task(retry_kwargs={'max_retries': 10})
def do_task_retry(*args, **kwargs):
    logger.debug("do_task_retry")

    return 1 / 0


@shared_task(
    default_retry_delay=3,
    retry_backoff=True,
    retry_kwargs={'max_retries': 10}
)
def do_task_retry_backoff_noautoretry(*args, **kwargs):
    logger.debug("do_task_retry_backoff")

    return 1 / 0


@shared_task(
    autoretry_for=(Exception,),
    default_retry_delay=3,
    retry_backoff=True,
    retry_kwargs={'max_retries': 10}
)
def do_task_retry_backoff(*args, **kwargs):
    logger.debug("do_task_retry_backoff autoretry")

    return 1 / 0


@shared_task(retry_kwargs={'max_retries': 3})
@retrying(10)
def do_task_retrying(*args, **kwargs):
    logger.debug("do_task_retrying")
    return 1 / 0


@shared_task
@retrying(3)
def proxy_task(**kwargs):
    method = kwargs.pop('method', '')
    url = kwargs.pop('url', '')

    logger.debug(kwargs)

    response = requests.request(method, url, **kwargs).json()

    return response


if __name__ == '__main__':
    # print(do_sync_task.apply_async(kwargs={'a': 1, 'b': 2}))
    print(AsyncToSync)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : kimi
# @Time         : 2024/12/9 13:14
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
import requests
import json

url = "https://kimi.moonshot.cn/api/chat/ctb5lpaflk1f1dda5mv0/completion/stream"

payload = json.dumps({
    "messages": [
        {
            "role": "user",
            "content": "hi"
        }
    ],
    "use_search": False,
    "extend": {
        "sidebar": True
    },
    "kimiplus_id": "kimi",
    "use_research": False,
    "use_math": False,
    "refs": [],
    "refs_file": []
})
headers = {
    'x-msh-session-id': '1729958401005741781',
    'Authorization': 'Bearer eyJhbGciOiJIUzUxMiIsInR5cCI6IkpXVCJ9.eyJpc3MiOiJ1c2VyLWNlbnRlciIsImV4cCI6MTczOTY4NDk3NywiaWF0IjoxNzMxOTA4OTc3LCJqdGkiOiJjc3RkYXNmZDBwODBpaGtkNTY4ZyIsInR5cCI6ImFjY2VzcyIsImFwcF9pZCI6ImtpbWkiLCJzdWIiOiJja2kwOTRiM2Flc2xnbGo2Zm8zMCIsInNwYWNlX2lkIjoiY2tpMDk0YjNhZXNsZ2xqNmZvMmciLCJhYnN0cmFjdF91c2VyX2lkIjoiY2tpMDk0YjNhZXNsZ2xqNmZvMzAifQ.uhEQ3sB6SJLR_Duuu4w-WilRsvllI611flQ_uQoI5ufm_GWtLLJfHZ8rE9-RS2YtkprtYovvEf1U1E6ybcL1Jg',
    'x-msh-platform': 'web',
    'x-msh-device-id': '7311290930975344143',
    'R-Timezone': 'Asia/Shanghai',
    'X-Traffic-Id': 'cki094b3aeslglj6fo30',
    'User-Agent': 'Apifox/1.0.0 (https://apifox.com)',
    'Content-Type': 'application/json'
}

response = requests.request("POST", url, headers=headers, data=payload)

print(response.text)



import os

# !/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : x
# @Time         : 2025/1/7 17:31
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.llm.clients import zhipuai_client
from meutils.schemas.openai_types import _ChatCompletion

import requests
import uuid

api_key = os.getenv("ZHIPUAI_API_KEY")

msg = [
    {
        "role": "user",
        "content": "中国队奥运会拿了多少奖牌"
    }
]

tool = "web-search-pro"


async def run_v4_sync():
    url = "https://open.bigmodel.cn/api/paas/v4/tools"
    request_id = str(uuid.uuid4())
    data = {
        "request_id": request_id,
        "tool": tool,
        "stream": False,
        "messages": msg
    }

    # resp = requests.post(
    #     url,
    #     json=data,
    #     headers={'Authorization': api_key},
    #     timeout=300
    # )
    # print(resp.content.decode())

    resp = await zhipuai_client.post(
        "/tools",
        body={
            "request_id": str(uuid.uuid4()),
            "tool": tool,
            "stream": False,
            "messages": msg
        },
        cast_to=object
    )
    return resp


file_object = zhipuai_client.files.create(file=Path("/Users/betterme/PycharmProjects/AI/MeUtils/meutils/llm/completions/rag/百炼系列手机产品介绍.docx"), purpose="file-extract")



if __name__ == '__main__':
    # r = arun(run_v4_sync())
    # arun(file_object)

    file_id = "1736243045_3771f3dfb394424885f24c3dc0583741"

    file_content = json.loads(zhipuai_client.files.content(file_id=file_id).content)["content"]

    file_content = json.loads(r.content)["content"]





from meutils.config_utils.lark_utils import get_next_token, aget_spreadsheet_values

feishu_url = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=Hcr2i8"


if __name__ == '__main__':
    from meutils.pipe import *

    df = arun(get_next_token(feishu_url=feishu_url))





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : http_utils
# @Time         : 2020/11/12 11:49 上午
# @Author       : yuanjie
# @Email        : meutils@qq.com
# @Software     : PyCharm
# @Description  : request cache https://mp.weixin.qq.com/s/9v9u4FhUtF8ivcu7Zj9NTQ

from tenacity import retry, stop_after_delay, stop_after_attempt, wait_fixed

# ME
from meutils.pipe import *


@retry(wait=wait_fixed(2),  # 重试之前等待2秒
       stop=stop_after_delay(7) | stop_after_attempt(3),  # 同时满足用 | 没毛病：重试7秒重试3次
       retry_error_callback=logger.error,
       reraise=True)
def request4retry(url, method='get', return_json=True, timeout=3, encoding=None, **kwargs):
    """None or {}"""
    # logger.info("Try")

    _kwargs = {}
    _kwargs['timeout'] = timeout
    _kwargs['headers'] = {
        'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) '
                      'Chrome/63.0.3239.132 Safari/537.36 QIHU 360SE '
    }
    r = requests.request(method, url, **{**_kwargs, **kwargs})
    r.encoding = encoding if encoding else r.apparent_encoding

    return r.json() if return_json else r


# def _create_retry_decorator(embeddings: OpenAIEmbeddings) -> Callable[[Any], Any]:
#     import openai
#
#     min_seconds = 4
#     max_seconds = 10
#     # Wait 2^x * 1 second between each retry starting with
#     # 4 seconds, then up to 10 seconds, then 10 seconds afterwards
#     return retry(
#         reraise=True,
#         stop=stop_after_attempt(embeddings.max_retries),
#         wait=wait_exponential(multiplier=1, min=min_seconds, max=max_seconds),
#         retry=(
#             retry_if_exception_type(openai.error.Timeout)
#             | retry_if_exception_type(openai.error.APIError)
#             | retry_if_exception_type(openai.error.APIConnectionError)
#             | retry_if_exception_type(openai.error.RateLimitError)
#             | retry_if_exception_type(openai.error.ServiceUnavailableError)
#         ),
#         before_sleep=before_sleep_log(logger, logging.WARNING),
#     )


@retry(wait=wait_fixed(3),  # 重试之前等待3秒
       stop=stop_after_delay(7) | stop_after_attempt(3),  # 同时满足用 | 没毛病：重试7秒重试3次
       retry_error_callback=lambda log: logger.error(log),
       reraise=True)
def request(url=None, json=None, parser=lambda x: x, encoding=None, **kwargs):
    """

    :param url:
    :param json:
    :param parser: None 的时候返回r，否则返回 parser(r.json())
    :param kwargs:
    :return:

        response = requests.request("GET", url, headers=headers, params=querystring)

    """
    method = 'post' if json is not None else 'get'  # 特殊情况除外
    logger.info(f"Request Method: {method}")
    headers = {
        'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) '
                      'Chrome/63.0.3239.132 Safari/537.36 QIHU 360SE '
    }
    r = requests.request(method, url, json=json, headers=headers)
    r.encoding = encoding if encoding else r.apparent_encoding

    if parser is None:
        return r
    return parser(r.json())


def is_url_alive(url):
    request = requests.head(url)
    return True if request.status_code == 200 else False


async def create_request(
        url,
        request,
        token: str = None,
        headers: Optional[dict] = None,
        post_kv: Optional[dict] = None
):
    token = token or 'This is a token'
    payload = request if isinstance(request, dict) else request.model_dump(exclude_none=True)

    headers = {
        'Authorization': f'Bearer {token}',
        'Cookie': token,
        **(headers or {})
    }
    async with httpx.AsyncClient(headers=headers, timeout=60) as client:
        response = await client.post(url=url, json=payload, **(post_kv or {}))
        if response.is_success:
            return response.json()
        response.raise_for_status()


if __name__ == '__main__':
    print(request4retry('xx'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : demo
# @Time         : 2024/3/22 15:55
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
import asyncio
import random
import time


async def producer(queue: asyncio.Queue, name: str):
    for i in range(10):
        # 模拟数据生成需要的时间
        await asyncio.sleep(random.random())
        item = (name, i)
        # 将数据放入队列
        await queue.put(item)
        print(f'生产者 {name} 生产了 {item}')
    # 生产完毕后发送结束信号
    await queue.put(None)


async def consumer(queue: asyncio.Queue, name: str):
    while True:
        # 从队列中获取数据
        item = await queue.get()
        if item is None:
            # 生产者发送了结束信号，终止循环
            break
        # 模拟数据处理需要的时间
        await asyncio.sleep(random.random())
        print(f'消费者 {name} 消费了 {item}')
        queue.task_done()


async def main():
    queue = asyncio.Queue()

    # 创建生产者和消费者协程
    producers = [producer(queue, f'P{i}') for i in range(2)]  # 两个生产者
    consumers = [consumer(queue, f'C{i}') for i in range(3)]  # 三个消费者

    # 启动所有协程
    await asyncio.gather(*producers, *consumers)

    # 等待队列被处理完毕
    await queue.join()


asyncio.run(main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : py_utils
# @Time         : 2022/7/25 上午10:14
# @Author       : yuanjie
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 主要是些内建的数据结构，tuple/list/set/dict/string

import json
import base64
from pydantic import BaseModel


# bytes/string


# 列表
def list_replace(l, mapping=None):
    if mapping is None:
        mapping = {}
    return [mapping.get(i, i) for i in l]  # [mapping[i] if i in mapping else i for i in l]


def list_difference(l1, l2):
    """列表差集，保持l1的顺序"""
    s = frozenset(l2)
    return [i for i in l1 if i not in s]


def list_intersection(l1, l2):
    """列表交集，保持l1的顺序"""

    s = frozenset(l2)
    return [i for i in l1 if i in s]


# padding

list4log = lambda ls: "\n\t" + "\n\t".join(ls)


# 字典


def bjson(json_data):
    if isinstance(json_data, (str,)):
        json_data = {"metadata": json_data}

    if isinstance(json_data, (dict, list)):
        return json.dumps(json_data, indent=4, ensure_ascii=False)

    elif isinstance(json_data, BaseModel):
        return json_data.model_dump_json(indent=4)


dict2json = bjson
json2dict = lambda s: json.loads(s.replace("'", '"'))


def dic2obj(dic):
    class Kwargs:
        pass

    kwargs = Kwargs()
    kwargs.__dict__ = dic
    return kwargs


def dict_explode(dic: dict):
    """
        d = {'a': range(10), 'b': range(20)}
        [(k, i) for k, v in d.items() for i in v]

    :param dic:
    :return:
    """
    return [(k, i) for k, v in dic.items() for i in v]


def multidict_reverse(multidict):
    """

    :param multidict:
    :return: {'k': [1, 2]} => {1: 'k', 2: 'k'}
    """
    return [(i, k) for k, v in multidict.items() for i in v]


def dict_merge(dicts):
    """
        pd.DataFrame(dicts).to_dict('list')
    @param dicts:
    @return:
    """
    dic = {}
    for d in dicts:
        for k, v in d.items():
            dic.setdefault(k, []).append(v)
    return dic


if __name__ == '__main__':
    # print(bjson('xx'))
    l2 = "async-result/{id}"
    l1 = "/async-result/if123456"#.removeprefix("/")
    print(list_difference(l1, l2))


# -*- coding: utf-8 -*-

from __future__ import absolute_import
from .types import TYPES
from .match import (
    image_match, font_match, document_match,
    video_match, audio_match, archive_match
)


def is_extension_supported(ext):
    """
    Checks if the given extension string is
    one of the supported by the file matchers.

    Args:
        ext (str): file extension string. E.g: jpg, png, mp4, mp3

    Returns:
        True if the file extension is supported.
        Otherwise False.
    """
    for kind in TYPES:
        if kind.extension == ext:
            return True
    return False


def is_mime_supported(mime):
    """
    Checks if the given MIME type string is
    one of the supported by the file matchers.

    Args:
        mime (str): MIME string. E.g: image/jpeg, video/mpeg

    Returns:
        True if the MIME type is supported.
        Otherwise False.
    """
    for kind in TYPES:
        if kind.mime == mime:
            return True
    return False


def is_image(obj):
    """
    Checks if a given input is a supported type image.

    Args:
        obj: path to file, bytes or bytearray.

    Returns:
        True if obj is a valid image. Otherwise False.

    Raises:
        TypeError: if obj is not a supported type.
    """
    return image_match(obj) is not None


def is_archive(obj):
    """
    Checks if a given input is a supported type archive.

    Args:
        obj: path to file, bytes or bytearray.

    Returns:
        True if obj is a valid archive. Otherwise False.

    Raises:
        TypeError: if obj is not a supported type.
    """
    return archive_match(obj) is not None


def is_audio(obj):
    """
    Checks if a given input is a supported type audio.

    Args:
        obj: path to file, bytes or bytearray.

    Returns:
        True if obj is a valid audio. Otherwise False.

    Raises:
        TypeError: if obj is not a supported type.
    """
    return audio_match(obj) is not None


def is_video(obj):
    """
    Checks if a given input is a supported type video.

    Args:
        obj: path to file, bytes or bytearray.

    Returns:
        True if obj is a valid video. Otherwise False.

    Raises:
        TypeError: if obj is not a supported type.
    """
    return video_match(obj) is not None


def is_font(obj):
    """
    Checks if a given input is a supported type font.

    Args:
        obj: path to file, bytes or bytearray.

    Returns:
        True if obj is a valid font. Otherwise False.

    Raises:
        TypeError: if obj is not a supported type.
    """
    return font_match(obj) is not None


def is_document(obj):
    """
    Checks if a given input is a supported type document.

    Args:
        obj: path to file, bytes or bytearray.

    Returns:
        True if obj is a valid document. Otherwise False.

    Raises:
        TypeError: if obj is not a supported type.
    """
    return document_match(obj) is not None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : config_manager
# @Time         : 2024/12/4 12:07
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

import nacos
from meutils.pipe import *


class ConfigManager(object):
    _instance = None
    _config: str = ""

    def __new__(cls, *args):
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    def __init__(self, namespace: str, data_id: str, group: str = "DEFAULT_GROUP", ):
        self.data_id = data_id
        self.group = group

        NACOS_URL = os.getenv("NACOS_URL")
        if NACOS_URL:
            server_addresses = NACOS_URL.split('@')[1]
            username, password = NACOS_URL.split('@')[0].split(":")

            # Nacos客户端配置
            self.client = nacos.NacosClient(
                server_addresses=server_addresses,  # Nacos服务器地址
                namespace=namespace,  # 命名空间
                username=username,  # 用户名
                password=password  # 密码
            )

    def init_config(self):
        """初始化配置并添加监听器"""
        # 获取初始配置
        config = self.client.get_config(self.data_id, self.group)
        logger.info(f"初始配置\n{config}")
        if config:
            self._config = config

        # 添加配置变更监听器
        self.client.add_config_watcher(self.data_id, self.group, self._config_changed_callback)

    def _config_changed_callback(self, args):
        """配置变更回调函数"""
        logger.debug(f"配置发生变更: {json.dumps(args, indent=4)}")
        # {'data_id': 'testdata', 'group': 'DEFAULT_GROUP', 'namespace': 'test', 'raw_content': 'sk-\nsk-',
        #  'content': 'sk-\nsk-'}
        self._config = args['content']

    @property
    def text(self) -> str:
        """获取当前配置: 原始内容"""
        return self._config or self.client.get_config(self.data_id, self.group)

    @property
    def json(self) -> Dict[str, Any]:
        text = self._config
        try:
            return json.loads(text)
        except json.decoder.JSONDecodeError as e:
            logger.warning(f"标准json 配置加载失败: {e}")

            logger.debug(f"尝试用json_repair解析")
            return json_repair.repair_json(text, return_objects=True)

    @property
    def yaml(self) -> Dict[str, Any]:
        text = self._config

        try:
            return yaml.safe_load(text)
        except Exception as e:
            logger.error(f"yaml 配置加载失败: {e}")

    @property
    def xml(self):
        return self._config

    @property
    def html(self):
        return self._config

    @property
    def properties(self):
        return self._config


if __name__ == '__main__':
    # 初始化配置管理器
    namespace = 'test'
    data_id = "testdata"

    namespace = 'prd'
    data_id = "siliconflow-api"

    manager = ConfigManager(namespace, data_id)
    # manager_ = Manager(namespace, data_id, group)

    print(manager.text)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : kolors
# @Time         : 2024/7/25 08:42
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://huggingface.co/spaces/gokaygokay/KolorsPlusPlus

from meutils.pipe import *
from meutils.schemas.openai_types import ImageRequest, ImagesResponse
from meutils.async_utils import sync_to_async
from meutils.decorators.retry import retrying
from meutils.config_utils.lark_utils import get_next_token_for_polling

from gradio_client import Client as _Client

Client = lru_cache(_Client)

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=72xGH2"


# os.getenv("HF_TOKEN")
@retrying(max_retries=3, title=__name__)
async def create_image(request: ImageRequest, token: Optional[str] = None):
    # token = token or await get_next_token_for_polling(FEISHU_URL)
    # src = "gokaygokay/KolorsPlusPlus"
    src = "Kwai-Kolors/Kolors"
    client = Client(src, download_files=False, hf_token=token)

    width = height = 1024
    if 'x' in request.size:
        width, height = map(int, request.size.split('x'))

    image = client.predict(
        prompt=request.prompt,
        ip_adapter_image=None,
        ip_adapter_scale=0.5,
        negative_prompt=request.negative_prompt,
        seed=0,
        randomize_seed=True,
        width=width,
        height=height,
        guidance_scale=request.guidance_scale,
        num_inference_steps=request.num_inference_steps,
        api_name="/infer"
    )

    return ImagesResponse(data=[image])


if __name__ == '__main__':
    r = ImageRequest(prompt='一张瓢虫的照片，微距，变焦，高质量，电影，拿着一个牌子，写着“可图”')
    # # with timer():
    # #     result = client.predict(
    # #         prompt="a cat",
    # #         negative_prompt=None,
    # #         api_name="/infer"
    # #     )
    # #     print(result)
    # # {'path': '/tmp/gradio/3a3fa0048e52b02169f4bb477d3c16e0a3fd7796/image.webp',
    # #  'url': 'https://kwai-kolors-kolors.hf.space/file=/tmp/gradio/3a3fa0048e52b02169f4bb477d3c16e0a3fd7796/image.webp',
    # #  'size': None, 'orig_name': 'image.webp', 'mime_type': None, 'is_stream': False,
    # #  'meta': {'_type': 'gradio.FileData'}}
    #
    with timer():
        arun(create_image(r))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : pdm
# @Time         : 2023/3/8 上午10:32
# @Author       : yuanjie
# @WeChat       : meutils
# @Software     : PyCharm


from xml.dom.minidom import parse
import xml.dom.minidom



class PDMHandler(object):
    PKG_ATTR_LIST = ["Name", "Code", "CreationDate", "Creator", "ModificationDate", "Modifier"]
    TBL_ATTR_LIST = ["Name", "Code", "CreationDate", "Creator", "ModificationDate", "Modifier",
                     "PhysicalOptions","Comment"]
    COL_ATTR_LIST = ["Name", "Code", "CreationDate", "Creator", "ModificationDate", "Modifier",
                     "DataType", "Length", "Column.Mandatory", "Comment"]
    IDX_ATTR_LIST = ["Name", "Code", "CreationDate", "Creator", "ModificationDate", "Modifier",
                     "PhysicalOptions", "Unique"]
    IDXCOL_ATTR_LIST = ["CreationDate", "Creator", "ModificationDate", "Modifier"]

    def __init__(self):
        return

    @staticmethod
    def parse(pdmfilename):
        """
        @brief       PDM文件导入函数
        @param[in]   pdmfilename       pdm文件名
        @return      pdm文件的DOM结构
        """
        return xml.dom.minidom.parse(pdmfilename)

    @staticmethod
    def __get_nodes_by_path(parent, xml_path):
        """
        @brief 按路径取节点核心函数,通过传入一个节点和一个路径
               (类文件系统路径)的字符串,获得相应的子结点(或集合).
               例1: __get_nodes_by_path(node,"a/b/c|3/d")
                 即获得node下a下b下第3个c节点下的所有d节点的一个
                 list.
               例2: __get_nodes_by_path(node,"a/b/c|3/d|2")
                 即获得node下a下b下第3个c节点下的第2个d节点.
                 注意:此处返回的不是list,而是节点
        @param[in]   parent DOM中的父节点
        @return      节点list或节点句柄
        """
        curr_node = parent
        for tag in xml_path.split("/")[0:-1]:
            tag_desc = tag.split("|")
            tag_name, tag_index = tag_desc[0], (int(tag_desc[1]) if len(tag_desc) == 2 else 0)
            child_nodes = []
            for child_node in curr_node.childNodes:
                if child_node.nodeName == tag_name:
                    child_nodes.append(child_node)
            if len(child_nodes) < tag_index + 1:
                return []
            curr_node = child_nodes[tag_index]
        # -- 最后一级路径特殊处理 -- #
        tag = xml_path.split("/")[-1]
        tag_desc = tag.split("|")
        tag_name, tag_index = tag_desc[0], (int(tag_desc[1]) if len(tag_desc) == 2 else None)
        child_nodes = []
        for child_node in curr_node.childNodes:
            if child_node.nodeName == tag_name:
                child_nodes.append(child_node)
        if tag_index == None:
            return child_nodes
        elif len(child_nodes) < tag_index + 1:
            return []
        else:
            curr_node = child_nodes[tag_index]
            return curr_node

    @staticmethod
    def __get_attrs_by_list(parent, attr_list):
        """
        @brief       取PDM属性(注意:此属性非xml属性）
                     背景:PDM文件中主要节点(o:Table,o:Column...)的具体属性
                          一般以子节点形式表示，而不以通常的XML属性表示.此
                          函数传入一个attr_list,输出一个字典表示的:
                          { "attr":"value" ...}
        @param[in]   parent    需要被取属性的节点句柄
        @param[in]   attr_list 一个字符串的list["attr1","attr2"...],代表要取的属性名集合
        @return      返回一个dict:{"attr1":"value" ...}
        """
        ret_dict = {}
        for attr in attr_list:
            ret_dict[attr] = ""
            for child in parent.childNodes:
                if child.nodeName == "a:" + attr:
                    ret_dict[attr] = child.childNodes[0].data
                    break
        return ret_dict

    @staticmethod
    def __get_pkgnodes_recursively(o_pkg):
        # -- 需要传入一个o:Model/o:Package节点 --#
        if o_pkg.nodeName != "o:Model" and o_pkg.nodeName != "o:Package":
            return []
        ret_list = []
        subpkgs = PDMHandler.__get_nodes_by_path(o_pkg, "c:Packages/o:Package")
        if subpkgs != None:
            for subpkg in subpkgs:
                ret_list.append(subpkg)
                ret_list = ret_list + PDMHandler.__get_pkgnodes_recursively(subpkg)
        else:
            return []
        return ret_list

    @staticmethod
    def getPkgNodes(hpdm):
        """
        @brief       获取pdm文件中所有的o:package的句柄list
        @param[in]   hpdm    待处理的pdm文件DOM句柄(可通过PDMHandler.parse取得)
        @return      返回pacakge的节点list:[pkg1,pkg2...],元素为DOM节点类型
        """
        ret_list = []
        try:
            o_mdl = PDMHandler.__get_nodes_by_path(hpdm, "Model/o:RootObject/c:Children/o:Model")[0]
            ret_list.append(o_mdl)
        except IndexError:
            print("ERROR:不是一个合法的pdm文件!")
            return []
        ret_list = ret_list + PDMHandler.__get_pkgnodes_recursively(o_mdl)
        return ret_list

    @staticmethod
    def getTblNodesInPkg(pkgnode):
        """
        @brief       获取指定o:Package下的所有o:Table的list
        @param[in]   pkgnode 待处理的o:Package节点(可通过PDMHandler.getPkgNodes取得)
        @return      返回o:Table的节点list:[tbl1,tbl2...],元素为DOM节点类型
        """
        return PDMHandler.__get_nodes_by_path(pkgnode, "c:Tables/o:Table")

    @staticmethod
    def getColNodesInTbl(tblnode):
        """
        @brief       获取指定o:Table下的所有o:Column的list
        @param[in]   tblnode 待处理的o:Table节点(可通过PDMHandler.getTblInPkg取得)
        @return      返回o:Column的节点list:[col1,col2...],元素为DOM节点类型
        """
        return PDMHandler.__get_nodes_by_path(tblnode, "c:Columns/o:Column")

    @staticmethod
    def getIdxNodesInTbl(tblnode):
        """
        @brief       获取指定o:Table下的所有o:Index的list
        @param[in]   tblnode 待处理的o:Table节点(可通过PDMHandler.getTblInPkg取得)
        @return      返回o:Index的节点list:[idx1,idx2...],元素为DOM节点类型
        """
        return PDMHandler.__get_nodes_by_path(tblnode, "c:Indexes/o:Index")

    @staticmethod
    def getIdxColNodesInIdx(idxnode):
        """
        @brief       获取指定o:Index下的所有o:IndexColumn的list
        @param[in]   idxnode 待处理的o:Index节点(可通过PDMHandler.getIdxNodesInTbl取得)
        @return      返回o:IndexColumn的节点list:[idxcol1,idxcol2...],元素为DOM节点类型
        """
        return PDMHandler.__get_nodes_by_path(idxnode, "c:IndexColumns/o:IndexColumn")

    @staticmethod
    def getPkgAttrs(pkgnode):
        """
        @brief       获取指定o:Package的属性(可取的属性参见PDMHandler.PKG_ATTR_LIST)
        @param[in]   pkgnode 待处理的o:Package节点(可通过PDMHandler.getPkgNodes取得)
        @return      返回一个字典dict:{"attr1":"value",...}
        """
        return PDMHandler.__get_attrs_by_list(pkgnode, PDMHandler.PKG_ATTR_LIST)

    @staticmethod
    def getTblAttrs(tblnode):
        """
        @brief       获取指定o:Table的属性(可取的属性参见PDMHandler.TBL_ATTR_LIST)
        @param[in]   tblnode 待处理的o:Table节点(可通过PDMHandler.getTblNodesInPkg取得)
        @return      返回一个字典dict:{"attr1":"value",...}
        """
        return PDMHandler.__get_attrs_by_list(tblnode, PDMHandler.TBL_ATTR_LIST)

    @staticmethod
    def getColAttrs(colnode):
        """
        @brief       获取指定o:Column的属性(可取的属性参见PDMHandler.COL_ATTR_LIST)
        @param[in]   colnode 待处理的o:Column节点(可通过PDMHandler.getColNodesInTbl取得)
        @return      返回一个字典dict:{"attr1":"value",...}
        """
        return PDMHandler.__get_attrs_by_list(colnode, PDMHandler.COL_ATTR_LIST)

    @staticmethod
    def getIdxAttrs(idxnode):
        """
        @brief       获取指定o:Index的属性(可取的属性参见PDMHandler.IDX_ATTR_LIST)
        @param[in]   idxnode 待处理的o:Index节点(可通过PDMHandler.getIdxNodesInTbl取得)
        @return      返回一个字典dict:{"attr1":"value",...}
        """
        return PDMHandler.__get_attrs_by_list(idxnode, PDMHandler.IDX_ATTR_LIST)

    @staticmethod
    def getIdxColAttrs(idxcolnode):
        """
        @brief       获取指定o:IndexColumn的属性(可取的属性参见PDMHandler.IDXCOL_ATTR_LIST)
        @param[in]   idxcolnode 待处理的o:IndexColumn节点(可通过PDMHandler.getIdxColNodesInIdx取得)
        @return      返回一个字典dict:{"attr1":"value",...}
        """
        ret_dict = PDMHandler.__get_attrs_by_list(idxcolnode, PDMHandler.IDXCOL_ATTR_LIST)
        refcol = PDMHandler.__get_nodes_by_path(idxcolnode, "c:Column/o:Column")
        try:
            refcolid = refcol[0].getAttribute("Ref")
        except IndexError:
            ret_dict["RefColCode"] = ""
            return ret_dict
        # -- 补充引用列插入字典 --#
        currnode = idxcolnode
        while (1):
            currnode = currnode.parentNode
            if currnode.tagName == "o:Table" or currnode == None:
                break
        if currnode == None:
            return []
        else:
            for col in PDMHandler.getColNodesInTbl(currnode):
                if col.getAttribute("Id") == refcolid:
                    ret_dict["RefColCode"] = PDMHandler.getColAttrs(col)["Code"]
        return ret_dict

    @staticmethod
    def getNodePath(node):
        """
        @brief       获取指定node的路径(模拟文件系统路径表示法)
        @param[in]   node 待处理的节点,类型为DOM的节点类型
        @return      返回一个字符串表示node的路径,形如"/foo/bar/node"
        """
        curr = node
        path_nodes = []
        while (1):
            if curr != None and curr.nodeName != "#document":
                path_nodes.append(curr.tagName)
            else:
                break
            curr = curr.parentNode
        path_nodes.reverse()
        path = "".join([slash + node for slash in '/' for node in path_nodes])
        return


if __name__ == '__main__':
    import pandas as pd
    import os
    dir = './o32/'
    for file in os.listdir(dir):
        filename = os.path.join(dir, file)
        print(filename)
        # filename = './o32/投资管理系统O3.2_风控.pdm'
        ph = PDMHandler.parse(filename)
        pkg_list = []
        for pkg in PDMHandler.getPkgNodes(ph):
            pkg_attrs = PDMHandler.getPkgAttrs(pkg)
            print("P:", pkg_attrs["Name"], pkg_attrs["Code"], pkg_attrs["Creator"])
            if not PDMHandler.getTblNodesInPkg(pkg):
                continue
            tbl_list = []
            for tbl in PDMHandler.getTblNodesInPkg(pkg):
                tbl_attrs = PDMHandler.getTblAttrs(tbl)
                # print(" T:", tbl_attrs["Name"], tbl_attrs["Code"], tbl_attrs["Creator"])
                # print("  T-PATH:", PDMHandler.getNodePath(tbl))
                col_list = []
                for col in PDMHandler.getColNodesInTbl(tbl):
                    col_attrs = PDMHandler.getColAttrs(col)
                    # print("  C:", col_attrs["Name"], col_attrs["Code"], col_attrs["DataType"], col_attrs["Length"],
                    #       col_attrs["Column.Mandatory"])
                    col_list.append(col_attrs)
                df = pd.DataFrame(col_list)
                df['tbl_Name'] = tbl_attrs["Name"]
                df['tbl_Code'] = tbl_attrs["Code"]
                df['tbl_Creator'] = tbl_attrs["Creator"]
                df['tbl_Comment'] = tbl_attrs["Comment"]
                tbl_list.append(df)
            df_tbl = pd.concat(tbl_list)
            df_tbl['pkg_Name'] = pkg_attrs["Name"]
            df_tbl['pkg_Code'] = pkg_attrs["Code"]
            df_tbl['pkg_Creator'] = pkg_attrs["Creator"]
            pkg_list.append(df_tbl)
        df_res = pd.concat(pkg_list)
        df_res = df_res[['pkg_Name', 'pkg_Code', 'pkg_Creator', 'tbl_Name', 'tbl_Code','tbl_Creator',
                         'tbl_Comment','Name', 'Code', 'CreationDate', 'Creator', 'ModificationDate',
                         'Modifier','DataType', 'Length', 'Column.Mandatory', 'Comment']]
        df_res.to_excel("./data/"+file.replace('pdm','xlsx'),index=False)

            # for idx in PDMHandler.getIdxNodesInTbl(tbl):
            #     idx_attrs = PDMHandler.getIdxAttrs(idx)
            #     print(idx_attrs)
            #     print("  I:", idx_attrs["Name"], idx_attrs["Code"], idx_attrs["Unique"])
            #     for idxcol in PDMHandler.getIdxColNodesInIdx(idx):
            #         idxcol_attrs = PDMHandler.getIdxColAttrs(idxcol)
            #         print("   IC:", idxcol_attrs["RefColCode"])

            # break



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : types
# @Time         : 2023/8/15 12:14
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
from os import PathLike

from meutils.pipe import *

StrPath = Union[str, PathLike]
StrOrPath = Union[str, PathLike]
StrOrCallableStr = Union[str, Callable[..., str]]

FileContent = Union[IO[bytes], bytes, PathLike]
FileTypes = Union[
    # file (or bytes)
    FileContent,
        # (filename, file (or bytes))
    Tuple[Optional[str], FileContent],
        # (filename, file (or bytes), content_type)
    Tuple[Optional[str], FileContent, Optional[str]],
        # (filename, file (or bytes), content_type, headers)
    Tuple[Optional[str], FileContent, Optional[str], Mapping[str, str]],
]
RequestFiles = Union[Mapping[str, FileTypes], Sequence[Tuple[str, FileTypes]]]


def is_list_of_strings(lst):
    return isinstance(lst, List) and all(isinstance(item, str) for item in lst)


def is_list_of_ints(lst):
    return isinstance(lst, List) and all(isinstance(item, int) for item in lst)





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : images
# @Time         : 2024/10/17 11:59
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 


from openai import AsyncOpenAI

from meutils.pipe import *
from meutils.config_utils.lark_utils import get_next_token_for_polling
from meutils.llm.openai_utils import to_openai_images_params
from meutils.schemas.image_types import ImageRequest, CogviewImageRequest
from meutils.notice.feishu import IMAGES, send_message as _send_message
from meutils.io.image import image2nowatermark_image

FEISHU_URL = "..."  # api
FEISHU_URL_FREE = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=siLmTk"

BASE_URL = "http://any2chat.chatfire.cn/glm/v1"

DEFAULT_MODEL = "cogview-3"
MODELS = {

}

send_message = partial(
    _send_message,
    title=__name__,
    url=IMAGES
)


async def generate(request: CogviewImageRequest, redirect_model: Optional[str] = None, api_key: Optional[str] = None):
    request.model = MODELS.get(request.model, DEFAULT_MODEL)

    api_key = api_key or await get_next_token_for_polling(FEISHU_URL_FREE)

    data = to_openai_images_params(request)
    for i in range(3):
        try:
            client = AsyncOpenAI(base_url=BASE_URL, api_key=api_key)
            response = await client.images.generate(**data)
            response.data[0].url = await image2nowatermark_image(response.data[0].url) or response.data[0].url
            return response
        except Exception as e:
            logger.error(e)
            if i > 2:
                send_message(f"生成失败: {e}\n\n{api_key}\n\n{request.model_dump_json(indent=4, exclude_none=True)}")


if __name__ == '__main__':
    from meutils.pipe import *

    arun(generate(CogviewImageRequest(model='cogview-3', prompt="a dog", size="1024x1024")))



    # response.data[0].url = await image2nowatermark_image(response.data[0].url)
